/***************************************************************************/
/**                                                                       **/
/**         GENDEFS.H - Definitions for M049_LIB and M049TASK             **/
/**                                                                       **/
/**  The File ONLY has to be included to generate a new library-file out  **/
/**  of the source-code. It must NOT be included into projects which use  **/
/**  the library.                                                         **/
/**                                                                       **/
/**             Project start    : 08.12.1996                             **/
/**             Last change      : 22.02.1999                             **/
/**                                                                       **/
/**                         Version: 2.D                                  **/
/**                         Author : JD                                   **/
/**                         Status : tested                               **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1996-99                        **/
/**                                                                       **/
/***************************************************************************/

#ifndef _general_definitions_
#define _general_definitions_

#include "m049_lib.h"

#define ANZ_MOD              11
#define M049_EEPROM_WORD_0   0x2131

typedef 	struct {
	ushort     task;
	ushort     fnr;
} USER_FUNC_STRUCT;

typedef struct {
	byte		data[8];
} CanDataType;

// this type was changed in version 2.B (req_size added)
typedef struct {
	M049Handle handle;
	ushort     req_size;
} ReadDataType;

typedef struct {
	M049Handle handle;
	byte       data[8];
} WriteDataType;

typedef struct {
	USER_FUNC_STRUCT serv_func;
	byte 				  usage;
	ulong  			  id;
} UnconfigMsgType;

// Numbers of Parameter
#define NO_OF_MICROSLOT_PAR  0
#define NO_OF_SRQHANDLE_PAR  2
#define NO_OF_SRQERROR_PAR   6
#define NO_OF_ERR_BUF_PAR    7
#define NO_OF_LED_STATUS_PAR 8
#define NO_OF_MASK_INT_PAR   10
#define NO_OF_DEBUG_PAR      12

// Function numbers of the cantask
#define NO_OF_START_FUNC    2
#define NO_OF_STOP_FUNC     3
#define NO_OF_CONFIG_FUNC   4
#define NO_OF_NEW_FUNC      5
#define NO_OF_MASK_FUNC     6
#define NO_OF_SEND_FUNC     7
#define NO_OF_REQUEST_FUNC  8
#define NO_OF_WRITE_FUNC    9
#define NO_OF_READ_FUNC     10
#define NO_OF_LED_FUNC      11
#define NO_OF_ID_FUNC       12
#define NO_OF_STOPMSG_FUNC  13
#define NO_OF_SRQ_FUNC      14
#define NO_OF_CLEAR_PROC    15
#define NO_OF_SRQEND_PROC   16
#define NO_OF_MSG_CNT_FUNC  17

// Number of Eeprom parameter
#define EEP_MODULTYP         0
#define EEP_INIT             1
#define EEP_TQ               2
#define EEP_TSEG1            3
#define EEP_TSEG2            4
#define EEP_TSJW             5
#define EEP_SPB              6
#define EEP_BUSTERM          7
#define EEP_DOUT             8
#define EEP_MSG_FORMAT       9

// Diagnose-Warnings
#define WRONG_SIZE           0x01  // different number of bytes received than wanted
#define SERV_NOT_INST        0x02  // the task with the service-function is not installed yet
#define CAN_MSG_WAITING      0x03  // trying to send a frame while the old one is still waiting
#define CAN_MSG_RECEIVED     0x04  // a message was received, whilst in the read or write function
#define MESSAGE_LOST         0x05  // a new message was received before the old one has been read out
#define NOT_SEND_IMMEDIATELY 0x06  // a message can't be send immediately because no free
											  // buffer the message handle is stored in the fifo and
											  // will be send if a message buffer will be available
// Diagnose-Error-Codes
#define NOT_ENOUGH_MEMORY    0x21  // trying to initialize more messageobjects than memory available
#define CAN_MODULE_NOT_READY 0x22  // The module is in reset state (hardware error)
#define ILLEGAL_IRQ          0x23  // the cantask has been installed with a wrong interrupt
#define WRONG_EEP_PAR        0x24  // EEPROM-Parameter is not a valid value
#define WRONG_PARAMETER      0x25  // not a valid parameter
#define TOO_MANY_MSG_OBJ     0x26  // trying to initialize more messageobjects than allowed
#define CAN_MSG_BUFFER_FULL  0x27  // trying to write more message objects directly into
											  // the buffers than allowed (max. 13)
#define ID_ALREADY_IN_USE    0x28  // trying to configure two messages with the same ID
#define WRONG_ID_MASK		  0x29  // parameter mask in call to function set_ID-mask illegal
#define WRONG_CAN_OBJ_TYPE   0x2A  // trying to use a transmission object in the function read
#define NO_FREE_BUFFER       0x2B  // all messagebuffer are busy
#define OBJ_OF_WRONG_TYPE    0x2C  // all passive messageobjects for buffer 15 must have same type
#define STATUS_ERROR         0x2D  // trying to configure the module a second time

#endif