/***************************************************************************/
/**                       M049_LIB, MODULAR-4/486                         **/
/**                                                                       **/
/**  This is the library for the M-CAN-1 module. It uses the functions    **/
/**  of the M049TASK.                                                     **/
/**  The functions can be used in both, a real time program running on    **/
/**  the MODULAR-4 or in a PC-program.                                    **/
/**                                                                       **/
/**                             Version 2.F.                              **/
/**                             date    23/07/99                          **/
/**                             author  J.D.                              **/
/**                                                                       **/
/**             (c) 1996-99 SORCUS Computer GmbH Heidelberg               **/
/**                                                                       **/
/***************************************************************************/

// History: see M049_LIB.H
// version 2.F: new m049_get_task_number to get the m049task-task-number that
//              belongs to a micro_slot
//              new m049_create_msg_object as alternative to m049_new_msg_object
//              for compiler that do no support byte alignment  
// version 2.E: new m049_init internal for SORCUS internal use instead of m049_bib_startup
// version 2.D: m049_get_diagnosis: reset of diagnosis only if it was != 0
// version 2.C: some #defines included to generate 32-Bit DLL for NT, 95, 98
// version 2.B: the m049task-function in m049_get_data is called with different
//              parameters now. The task-function generates warning 1 instead
//              of the library.
// version 2.A: variable taskversion added: to avoid conflicts between
//              new library and old M049TASK the driver version is checked
//              in the new library functions (initialized in m049_bib_startup)

/* If you want to generate a library for a different compiler or platform
   the following #defines have to exist:
   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8RT has to be included.
   If the library shall be used on a pc #define ML8 has to be included.
   If you work with Windows 3.x, #define _WIN16 has to be included.
   If you work with Windows NT, 95 or 98, #define _WIN32 has to be included.
   For WIN16 and WIN32 programs define _WINDOWS.
*/

#include "m049int.h"

#ifdef _WINDOWS
   #include <windows.h>
#endif

#ifdef _WIN16
   HANDLE   hInst;
   #pragma argsused
   int far pascal LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
      HANDLE  hInstance;                   /* instance of DLL */
      WORD    wDataSeg;                    /* data segment */
      WORD    wHeapSize;                   /* size of heap */
      LPSTR   lpszCmdLine;                 /* pointer to command line */
      {
         hInst = hInstance;
         if(wHeapSize > 0)
            UnlockData(0);
         return(1);
      }
#endif 

#ifdef ML8RT
   STATIC ushort tasknum[ANZ_MOD];
   STATIC ushort taskversion[ANZ_MOD];
#else
   STATIC USHORT tasknum[8][ANZ_MOD];   /* new in version 2.F: 1 array */
   STATIC USHORT taskversion[8][ANZ_MOD];            /* for each board */
#endif

/***************************************************************************/
/*                       Init of the module library                        */
/* Check for M-CAN-1 modules and m049task.exe.                             */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_bib_startup(void)
{
   ushort install_nr, taskbuf[ANZ_MOD], task_cnt, modul_cnt, slotbuf[ANZ_MOD], card;
   byte   micro_slot;
   union {
      ulong    result;
      byte     version[4];
   } prog_info;

   task_cnt = 0;
   modul_cnt = 0;

#ifdef ML8RT
   ml8rt_get_task_number(M049TASK_PROG_NO, &task_cnt);
   if (task_cnt == 0)                  // no m049task installed
      return (M049_INSTALL_ERROR);
   for (micro_slot=1;micro_slot<ANZ_MOD;micro_slot++) {
      slotbuf[micro_slot] = ml8rt_read_eeprom_copy(micro_slot, 0);
      if ((slotbuf[micro_slot] & 0x00FF) == (M049_EEPROM_WORD_0 & 0x00FF))
         modul_cnt++;                  // count number of M-CAN-1 modules
      else
         slotbuf[micro_slot] = 0;
   }
   if (task_cnt != modul_cnt)          // number of modules differs from
      return (M049_INSTALL_ERROR);     // number of installed m049task
   for (modul_cnt=1;modul_cnt < (task_cnt + 1);modul_cnt++) {
      install_nr = modul_cnt;
      taskbuf[modul_cnt] = ml8rt_get_task_number(M049TASK_PROG_NO, &install_nr);
   }
   task_cnt = 1;
   // assign a task number to every micro slot with M-CAN-1:
   for (micro_slot=1;micro_slot<ANZ_MOD;micro_slot++)
      if (slotbuf[micro_slot]) {
         tasknum[micro_slot] = taskbuf[task_cnt++];
         ml8rt_get_task_info(tasknum[micro_slot],4,&prog_info.result);
         // save version code as 0x0201 = Vers. '2', Rev. 'A'
         taskversion[micro_slot] = (prog_info.version[2] - '0') << 8;
         taskversion[micro_slot] += prog_info.version[3] - 'A' + 1;
      }
      else
         tasknum[micro_slot] = 0;
#else
   card = ml8_get_selected_card();
   ml8_get_task_number(M049TASK_PROG_NO, &task_cnt);
   if (task_cnt == 0)                  // no m049task installed
      return (M049_INSTALL_ERROR);
   for (micro_slot=1;micro_slot<ANZ_MOD;micro_slot++) {
      slotbuf[micro_slot] = ml8_read_eeprom_copy(micro_slot, 0);
      if ((slotbuf[micro_slot] & 0x00FF) == (M049_EEPROM_WORD_0 & 0x00FF))
         modul_cnt++;                  // count number of M-CAN-1 modules
      else
         slotbuf[micro_slot] = 0;
   }
   if (task_cnt != modul_cnt)          // number of modules differs from
      return (M049_INSTALL_ERROR);     // number of installed m049task

   for (modul_cnt=1;modul_cnt < (task_cnt + 1);modul_cnt++) {
      install_nr = modul_cnt;
      taskbuf[modul_cnt] = ml8_get_task_number(M049TASK_PROG_NO, &install_nr);
   }
   task_cnt = 1;
   // assign a task number to every micro slot with M-CAN-1:
   for (micro_slot=1;micro_slot<ANZ_MOD;micro_slot++)
      if (slotbuf[micro_slot]) {
         tasknum[card][micro_slot] = taskbuf[task_cnt++];
         ml8_get_task_info(tasknum[card][micro_slot],4,&prog_info.result);
         // save version code as 0x0201 = Vers. '2', Rev. 'A'
         taskversion[card][micro_slot] = (prog_info.version[2] - '0') << 8;
         taskversion[card][micro_slot] += prog_info.version[3] - 'A' + 1;
      }
      else
         tasknum[card][micro_slot] = 0;
#endif
   return (0);
}

/***************************************************************************/
/*                          Initialize CAN module                          */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_config_module (byte micro_slot, M049ConfigType *config)
{
   ushort insize, error, outsize, indata, card;

   error = 0;
   insize = 0;
   outsize = sizeof(*config);
#ifdef ML8RT
   if (tasknum[micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8rt_write_par_byte(tasknum[micro_slot],NO_OF_MICROSLOT_PAR,micro_slot);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_CONFIG_FUNC, &outsize,
                   config, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   if (tasknum[card][micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8_write_par_byte(tasknum[card][micro_slot],NO_OF_MICROSLOT_PAR,micro_slot);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_CONFIG_FUNC, outsize,
                  config,insize,&insize,&indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                          New Message Object                             */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_config_msg_object(byte micro_slot,M049Handle *handle,M049MsgType *config)
{
   ushort outsize, insize, error, card;
   M049Handle h;

   outsize = sizeof(*config);
   insize = sizeof(h);
   error = 0;
#ifdef ML8RT
   if (tasknum[micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_NEW_FUNC, &outsize,
                   config, insize, &insize, &h, &error);
#else
   card = ml8_get_selected_card();
   if (tasknum[card][micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_NEW_FUNC, outsize,
                  config, insize, &insize, &h, (byte*)&error);
#endif
   *handle = h;
   return(error);
}

/***************************************************************************/
/* Create new message object                                               */
/***************************************************************************/

ML10API ushort MLXEXPORT m049_create_msg_object(byte micro_slot, M049Handle *handle,
												ulong id, byte control_flags,
												ushort serv_task, ushort serv_func)
{
	M049MsgType config;

	config.id = id;
	config.control_flags = control_flags;
	config.serv_task = serv_task;
	config.serv_func = serv_func;
	return(m049_config_msg_object(micro_slot, handle, &config));
}

/***************************************************************************/
/*                       Writes the acceptance mask filter                 */
/* recommended only if more messages than defined should be received       */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_set_acceptance_mask (byte micro_slot, ulong *mask, byte usage,
                                        ushort serv_task, ushort serv_func)
{
   ushort insize, error, outsize, card;
   UnconfigMsgType outdata;
   ulong indata;

   outdata.id = *mask;
   outdata.usage = usage;
   outdata.serv_func.task = serv_task;
   outdata.serv_func.fnr  = serv_func;
   outsize = sizeof(outdata);
   insize = sizeof(indata);
   error = 0;
#ifdef ML8RT
   if (tasknum[micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_MASK_FUNC, &outsize, &outdata,
                   insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   if (tasknum[card][micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_MASK_FUNC,  outsize, &outdata,
                  insize, &insize, &indata,(byte*)&error);
#endif
   *mask = indata;
   return(error);
}

/***************************************************************************/
/*                               Start CAN Communication                   */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_start_com (byte micro_slot)
{
   ushort insize, error, outsize, outdata, indata, card;

   outsize = 0;
   insize = 0;
   error = 0;
#ifdef ML8RT
   if (tasknum[micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_START_FUNC, &outsize, &outdata,
                   insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   if (tasknum[card][micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_START_FUNC, outsize, &outdata,
                  insize, &insize, &indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                           Stop CAN Communication                        */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_stop_com (byte micro_slot)
{
   ushort insize, error, outsize, outdata, indata, card;

   outsize = 0;
   insize = 0;
   error = 0;
#ifdef ML8RT
   if (tasknum[micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_STOP_FUNC, &outsize, &outdata,
                   insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   if (tasknum[card][micro_slot] == 0)
      return(M049_INVALID_SLOT);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_STOP_FUNC,  outsize, &outdata,
                  insize, &insize, &indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                           Write Object data                             */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_set_data (byte micro_slot, M049Handle handle,
                                         byte *data, byte size)
{
   ushort         outsize, insize, indata, error, card;
   WriteDataType  outdata;

   if (size > 8)
      return(M049_WRONG_PAR);
   error = 0;
   insize = 0;
   outsize = sizeof(handle) + size;
   outdata.handle = handle;
   // copy 8 bytes, do not consider size for copying
   *(ulong*)&outdata.data[0] = *(ulong*)&data[0];
   *(ulong*)&outdata.data[4] = *(ulong*)&data[4];
#ifdef ML8RT
   ml8rt_call_func (tasknum[micro_slot], NO_OF_WRITE_FUNC,  &outsize,
                  &outdata, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_WRITE_FUNC, outsize,
                  &outdata, insize, &insize, &indata, (byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                         Write Object on CAN-bus                         */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_send_data (byte micro_slot, M049Handle handle)
{
   ushort outsize, insize, indata, error, card;

   outsize = sizeof(handle);
   insize = 0;
   error = 0;
#ifdef ML8RT
   ml8rt_call_func(tasknum[micro_slot], NO_OF_SEND_FUNC, &outsize,
                   &handle, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_SEND_FUNC, outsize,
                  &handle, insize, &insize,&indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                           Read Object data                              */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_get_data (byte micro_slot, M049Handle handle,
                             byte *data_var, byte *size_var)
{
   ushort          outsize, insize, error, i, card;
   CanDataType     msg_data;
   ReadDataType    outdata;

   if (*size_var > 8)
      return(M049_WRONG_PAR);
   error = 0;
   insize = sizeof(msg_data);
   outdata.handle = handle;
   // in M049TASK version 2.B the prototype for the function that is called
   // here has changed. The WRONG_SIZE warning is generated in the task now.
#ifdef ML8RT
   if (taskversion[micro_slot] > 0x0201) {     // M049TASK >= 2.B
      outsize = sizeof(outdata);
      outdata.req_size = *size_var;
      ml8rt_call_func (tasknum[micro_slot], NO_OF_READ_FUNC, &outsize,
                       &outdata, insize, &insize, &msg_data, &error);
      if (!error) {
         for (i=0;((i<*size_var) && (i<insize));i++)
            data_var[i] = msg_data.data[i];
         if (*size_var != insize)
            *size_var = (byte)insize;
      }
   }
   else {                                      // M049TASK < 2.B
      outsize = sizeof(handle);
      ml8rt_call_func (tasknum[micro_slot], NO_OF_READ_FUNC, &outsize,
                       &outdata.handle, insize, &insize, &msg_data, &error);
      if (!error) {
         for (i=0;((i<*size_var) && (i<insize));i++)
            data_var[i] = msg_data.data[i];
         if (*size_var != insize) {
            *size_var = (byte)insize;
            ml8rt_write_par_byte(tasknum[micro_slot],NO_OF_ERR_BUF_PAR,WRONG_SIZE);
         }
      }
   }
#else
   card = ml8_get_selected_card();
   if (taskversion[card][micro_slot] > 0x0201) {     // M049TASK >= 2.B
      outsize = sizeof(outdata);
      outdata.req_size = *size_var;
      ml8_call_func (tasknum[card][micro_slot], NO_OF_READ_FUNC,  outsize,
                     &outdata, insize, &insize, &msg_data, (byte*)&error);
      if (!error) {
         for (i=0;((i<*size_var) && (i<insize));i++)
            data_var[i] = msg_data.data[i];
         if (*size_var != insize)
            *size_var = (byte)insize;
      }
   }
   else {                                      // M049TASK < 2.B
      outsize = sizeof(handle);
      ml8_call_func (tasknum[card][micro_slot], NO_OF_READ_FUNC,  outsize,
                     &outdata.handle, insize, &insize, &msg_data, (byte*)&error);
      if (!error) {
         for (i=0;((i<*size_var) && (i<insize));i++)
            data_var[i] = msg_data.data[i];
         if (*size_var != insize) {
            *size_var = (byte)insize;
            ml8_write_par_byte(tasknum[card][micro_slot],NO_OF_ERR_BUF_PAR,WRONG_SIZE);
         }
      }
   }
#endif
   return(error);
}

/***************************************************************************/
/*                          Read Object from CAN-bus                       */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_send_data_request (byte micro_slot, M049Handle handle)
{
   ushort outsize, insize, indata, error, card;

   outsize = sizeof(handle);
   insize = 0;
   error = 0;
#ifdef ML8RT
   ml8rt_call_func(tasknum[micro_slot], NO_OF_REQUEST_FUNC, &outsize,
                   &handle, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_REQUEST_FUNC,  outsize,
                  &handle, insize,&insize,&indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                           Read Object id                                */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_get_id (byte micro_slot, M049Handle handle, ulong *id)
{
   ushort insize, error, outsize, card;
   ulong  indata;

   error = 0;
   outsize = sizeof(handle);
   insize = sizeof(indata);
#ifdef ML8RT
   ml8rt_call_func(tasknum[micro_slot], NO_OF_ID_FUNC, &outsize, &handle,
                   insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_ID_FUNC, outsize, &handle,
                  insize, &insize, &indata, (byte*)&error);
#endif
   *id = indata;
   return(error);
}

/***************************************************************************/
/*                          Read Object interrupt count                    */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_get_msg_count (byte micro_slot, M049Handle handle, ulong *count)
{
   ushort insize, error, outsize, card;
   ulong  indata;

   // this function is implemented in m049task from version 2.B
   error = 0;
   outsize = sizeof(handle);
   insize = sizeof(indata);
#ifdef ML8RT
   if (taskversion[micro_slot] < 0x0202)
      return(M049_FUNC_NOT_IMPLEMENTED);
   ml8rt_call_func(tasknum[micro_slot], NO_OF_MSG_CNT_FUNC, &outsize,
                   &handle, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   if (taskversion[card][micro_slot] < 0x0202)
      return(M049_FUNC_NOT_IMPLEMENTED);
   ml8_call_func (tasknum[card][micro_slot], NO_OF_MSG_CNT_FUNC, outsize,
                  &handle, insize, &insize, &indata, (byte*)&error);
#endif
   *count = indata;
   return(error);
}

/***************************************************************************/
/*                          Read Object from CAN-bus                       */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_clear_msg (byte micro_slot, M049Handle handle)
{
   ushort outsize, insize, indata, error, card;

   outsize = sizeof(handle);
   insize = 0;
   error = 0;
#ifdef ML8RT
   ml8rt_call_func(tasknum[micro_slot], NO_OF_STOPMSG_FUNC, &outsize,
                   &handle, insize, &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_STOPMSG_FUNC,  outsize,
                  &handle, insize,&insize,&indata,(byte*)&error);
#endif
   return(error);
}

/***************************************************************************/
/*                           Switch module led                             */
/***************************************************************************/

ML10API void MLXEXPORT  m049_set_dout (byte micro_slot, byte nr, byte onoff)
{
   ushort insize, indata, outsize, error, card;

#ifdef _WIN32
   #pragma pack(push, SORCPACK, 1)     // for the 32-bit DLL (MSVC)
#else
   #pragma -a-                         // for dos, rt, win (bc)
#endif
   struct datastruct {  byte nr;
                        byte onoff;
   }data;
#ifdef _WIN32
   #pragma pack(pop, SORCPACK)
#else
   #pragma -a
#endif

   if ((nr < 2) && (onoff < 2)) {
      data.nr = nr;
      data.onoff = onoff;
      outsize = sizeof(data);
      insize = 0;
#ifdef ML8RT
   ml8rt_call_func(tasknum[micro_slot], NO_OF_LED_FUNC, &outsize, &data,
                   0 , &insize, &indata, &error);
#else
   card = ml8_get_selected_card();
   ml8_call_func (tasknum[card][micro_slot], NO_OF_LED_FUNC, outsize, &data,
                  0 ,&insize,&indata,(byte*)&error);
#endif
   }
}

/***************************************************************************/
/*                           Get DOUT status                               */
/***************************************************************************/

ML10API byte MLXEXPORT  m049_get_dout (byte micro_slot, byte nr)
{
   // this function is implemented in m049task from version 2.A
#ifdef ML8RT
   return(ml8rt_read_par_byte(tasknum[micro_slot], (ushort)(NO_OF_LED_STATUS_PAR+nr)));
#else
   ushort card;

   card = ml8_get_selected_card();
   return(ml8_read_par_byte(tasknum[card][micro_slot], (ushort)(NO_OF_LED_STATUS_PAR+nr)));
#endif
}

/***************************************************************************/
/*                          Read Object from CAN-bus                       */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_clear_configuration(byte micro_slot)
{
   // this function is implemented in m049task from version 2.A
#ifdef ML8RT
   if (taskversion[micro_slot] < 0x0201)
      return(M049_FUNC_NOT_IMPLEMENTED);
   ml8rt_call_proc(tasknum[micro_slot], NO_OF_CLEAR_PROC);
#else
   ushort card;

   card = ml8_get_selected_card();
   if (taskversion[card][micro_slot] < 0x0201)
      return(M049_FUNC_NOT_IMPLEMENTED);
   ml8_call_proc (tasknum[card][micro_slot], NO_OF_CLEAR_PROC);
#endif
   return(0);
}

/***************************************************************************/
/*            get handle after srq: only in PC-programs                    */
/***************************************************************************/

#ifdef ML8RT
   #pragma argsused
#endif
ML10API ushort MLXEXPORT  m049_evaluate_srq(byte *micro_slot, M049Handle *handle)
{
   ushort error;

#ifndef ML8RT
   ushort card;
   union {
      ushort w;
      byte   b[2];
   } srq;

   error = 0;
   *handle = 0;
   if (!ml8_message_available())
      return(0);
   srq.w = ml8_get_message();
    card = ml8_get_selected_card();
   if (srq.b[0] == 0x80) {             // SRQ from the cantask
      *micro_slot = srq.b[1] & 0x0F;   // micro_slot that requests SRQ
      error = ml8_read_par_byte(tasknum[card][*micro_slot],NO_OF_SRQERROR_PAR);
      ml8_read_par_dword(tasknum[card][*micro_slot],NO_OF_SRQHANDLE_PAR,(ulong*)handle);
   }
#else
   error = 0;
   *handle = 0;
#endif
   return(error);
}

/***************************************************************************/
/*         enable communication at the end of the PC-service-procedure     */
/***************************************************************************/

#ifdef ML8RT
   #pragma argsused
#endif
ML10API void MLXEXPORT  m049_end_of_srq(byte micro_slot)
{
#ifndef ML8RT
   ushort card;

    card = ml8_get_selected_card();
   if (taskversion[card][micro_slot] >= 0x0201) // M049TASK = 2.A or higher
      ml8_call_proc(tasknum[card][micro_slot],NO_OF_SRQEND_PROC);
#endif
}

/***************************************************************************/
/* enable interrupt masking for srq service                                */
/***************************************************************************/

#ifdef ML8RT
   #pragma argsused
#endif
ML10API void MLXEXPORT  m049_enable_int_mask(byte micro_slot)
{
#ifndef ML8RT
   ushort card;

    card = ml8_get_selected_card();
   if (taskversion[card][micro_slot] >= 0x0201) // M049TASK = 2.A or higher
      ml8_write_par_byte(tasknum[card][micro_slot],NO_OF_MASK_INT_PAR, 1);
#endif
}

/***************************************************************************/
/*            get status-information from the task                         */
/***************************************************************************/

ML10API ushort MLXEXPORT  m049_get_diagnosis(byte micro_slot)
{
   byte   status;

#ifdef ML8RT
   status = ml8rt_read_par_byte(tasknum[micro_slot],NO_OF_ERR_BUF_PAR);
   if (status)
      ml8rt_write_par_byte(tasknum[micro_slot],NO_OF_ERR_BUF_PAR,0);
#else
   ushort card;

   card = ml8_get_selected_card();
   status = ml8_read_par_byte(tasknum[card][micro_slot],NO_OF_ERR_BUF_PAR);
   if (status)
      ml8_write_par_byte(tasknum[card][micro_slot],NO_OF_ERR_BUF_PAR,0);
#endif
   return(status);
}

/***************************************************************************/
/* init_internal - can be used instead of bib_startup (SORCUS internal)    */
/* requires that the task number of m049task is equal to its micro_slot    */
/***************************************************************************/

ML10API void MLXEXPORT m049_init_internal()
{
   USHORT i;
   union {
      ULONG    result;
      USHORT   w[2];
      UCHAR    version[4];
   } prog_info;

#ifdef ML8RT
   for (i=1;i<ANZ_MOD;i++) {
      ml8rt_get_task_info(i, 4, &prog_info.result);
      if (prog_info.w[0] == M049TASK_PROG_NO) {
         tasknum[i] = i;
         // save version code as 0x0201 = Vers. '2', Rev. 'A'
         taskversion[i] = (prog_info.version[2] - '0') << 8;
         taskversion[i] += prog_info.version[3] - 'A' + 1;
      }
      else {
         tasknum[i] = 0;
         taskversion[i] = 0;
      }
   }
#else
   ushort card;

   card = ml8_get_selected_card();
   for (i=1;i<ANZ_MOD;i++) {
      ml8_get_task_info(i, 4, &prog_info.result);
      if (prog_info.w[0] == M049TASK_PROG_NO) {
         tasknum[card][i] = i;
         // save version code as 0x0201 = Vers. '2', Rev. 'A'
         taskversion[card][i] = (prog_info.version[2] - '0') << 8;
         taskversion[card][i] += prog_info.version[3] - 'A' + 1;
      }
      else {
         tasknum[card][i] = 0;
         taskversion[card][i] = 0;
      }
   }
#endif
}

/***************************************************************************/
/* get_task_number - get the task number of m049task.exe that belongs to a */
/* certain module slot, returns 0 if no module on that slot                */
/***************************************************************************/

ML10API ushort MLXEXPORT m049_get_task_number(byte micro_slot)
{
#ifdef ML8RT
   return(tasknum[micro_slot]);
#else
   ushort card;

   card = ml8_get_selected_card();
   return(tasknum[card][micro_slot]);
#endif
}

