/***************************************************************************/
/**                                                                       **/
/**         M049_LIB.H - Prototypes and definitions for M049_LIB.         **/
/**                                                                       **/
/**             Project start    : 08.12.1996                             **/
/**             Last change      : 23.07.1999                             **/
/**                                                                       **/
/**                         Version: 2.F                                  **/
/**                         Author : JD                                   **/
/**                         Status : tested                               **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1996-99                        **/
/**                                                                       **/
/***************************************************************************/
/**                                                                       **/
/**                       History (for M049_LIB and M049TASK)             **/
/**  Version/Date     reason                                              **/
/**   2.F (23.07.99):  bug fixes                                          **/
/**                  - In M049TASK V. 2.E receiving telegrams with 8 bytes**/
/**                    data lenght caused error that service procedure was**/
/**                    not called anymore.                                **/
/**                  - The library did not support various M-CAN-1 modules**/
/**                    on more than one base board.                       **/     
/**                  new features                                         **/
/**                  - new function m049_get_task_number added. It allows **/
/**                    to find out which task number of m049task.exe      **/
/**                    belongs to which micro-slot.                       **/ 
/**                  - new function m049_create_msg_object. It does       **/
/**                    exactely the same as m049_config_msg_obj but the   **/
/**                    parameters are passed in a way that it can be used **/
/**                    with Visual Basic.                                 **/
/**   2.E (03.05.99):  bug fixes                                          **/
/**                  - if m049_config_msg_object detects an error the     **/
/**                    message object that should be configured will now  **/
/**                    be removed from the dynamic list of configured     **/
/**                    message objects. Before it was in the list but     **/
/**                    marked invalid.                                    **/
/**                  - function decorations added in this header, this    **/
/**                    caused some problems under WIN32 with MFC          **/
/**                  new features                                         **/
/**                  - after m049_clear_configuration new message objects **/
/**                    will now not occupy new memory they will be written**/
/**                    into the cleared memory space                      **/
/**                  - m049task with further optimizations for speed      **/
/**                  - 2 new functions added to m049task for internal test**/
/**                  - parameter 13 defined new (last error code)         **/
/**                  - m049_init_internal added to the library (can be    **/
/**                    instead of m049_bib_startup, if slot = task)       **/
/**   2.D (22.02.99):  bug fix                                            **/
/**                  - m049_config_msg_object returned the wrong value if **/
/**                    too many objects are declared with exclusive buffer**/
/**   2.C (18.01.99):  32-Bit DLL added                                   **/
/**   2.B (16.07.98):  bug fixes                                          **/
/**                  - in certain casees it could happen that M049TASK    **/
/**                    will stop working (because the module does not     **/
/**                    generated interrupts anymore).                     **/
/**                  - m049_get_data: warning 1 (wrong size) overwrites   **/
/**                    warning 5 (message lost). This is changed because  **/
/**                    a lost message is more important than a wrong size **/
/**                  new features                                         **/
/**                  - new funcion m049_get_msg_count                     **/
/**                  - possibility to measure the interrupt performance   **/
/**   2.A (26.02.98):  bug fixes                                          **/
/**                  - m049_set_acceptance_mask: format of the value      **/
/**                    returned in 'mask' changed. (see readme.txt)       **/
/**                  - extern "C" for C++ was not correct in M049_LIB.H   **/
/**                  - definition of M049TASK_FLAGS changed               **/
/**                  - switch of the terminator-resistance did not work   **/
/**                    (it was always on in the former versions)          **/
/**                  - active-only message objects were always active and **/
/**                    passive.                                           **/
/**                  - The definition M049_EEPROM_WORD_0 was removed from **/
/**                    the header-file because the value of the EEPROM-   **/
/**                    word 0 is hardware dependent and without any use   **/
/**                    for programs which use M049TASK.                   **/
/**                  new features                                         **/
/**                  - new funcion m049_get_dout                          **/
/**                  - new funcion m049_clear_configuration               **/
/**                  - new function m049_end_of_srq                       **/
/**                  - new function m049_enable_int_mask                  **/
/**                  - dynamical configuration of message objects at      **/
/**                    any time (the parameter 'messages' passed to       **/
/**                    m049_config_module is without significance now!)   **/
/**                  - send objects can get the property M049_DISABLE_INT **/
/**                  - M049_SRQ_SERVICE_FUNC (=task function 14) defined  **/
/**                  - m049_get_data and m049_set_data checks the correct-**/
/**                    ness of size_var now (before the program might     **/
/**                    crash with invalid parameter values).              **/
/**                  - more parameter checking in the non-time-critical   **/
/**                    functions                                          **/
/**                  - more intensive checking for lost messages          **/
/**                                                                       **/
/**   1.C (24.03.97): bug fixes of for former versions                    **/
/**                  - Extended can message format did not work. The      **/
/**                    value of the define _M049_SEND changed. Not        **/
/**                    compatible with beta version 1.B                   **/
/**                  - When using M049TASK function 14 as error service   **/
/**                    function an error was never reset.                 **/
/**                  - m049_get_data returned as many bytes as it was     **/
/**                    specified in size_var, even if less bytes were     **/
/**                    received.                                          **/
/**                                                                       **/
/**   1.B (20.02.97): beta release                                        **/
/***************************************************************************/

#ifndef _m049_lib_
#define _m049_lib_

#ifdef ML8RT
   #include "ml8rtbib.h"
   #include "dos.h"
   #define  ML10API
   #define  MLXEXPORT     far pascal
#else
   #ifdef _WIN32
      #include "MLXW32.H"
   #else
      #include "ML8BIB.H"
      #define  MLXEXPORT  EXPORT
   #endif
#endif

#ifdef BORLAND_PASCAL
   #define STATIC static                /* for definition of arrays */
#else
   #define STATIC static far            /* for definition of arrays */
#endif

/****************************************************************************/
// Variable type definitions used by M049_LIB:

#ifndef byte
   #define byte unsigned char
#endif
#ifndef ulong
   #define ulong unsigned long
#endif

#ifdef ML8RT
   #ifndef ushort
      #define ushort unsigned int
   #endif
#else
   #ifndef ushort
      #define ushort unsigned short
   #endif
#endif

#define M049Handle       ulong         // message handle

// use byte alignment for the structs
#ifdef _WIN32
   #pragma pack(push, SORCPACK, 1)     // for the 32-bit DLL (MSVC)
#else
   #pragma -a-                         // for dos, rt, win (bc)
#endif
typedef struct {
   ushort   messages;              // Number of configured messageobjects
   ushort   serv_task;             // Task of error handling function
   ushort   serv_func;             // Number of error handling function
} M049ConfigType;

typedef struct {
   ulong    id;                    // Can Message Identifier
   byte     control_flags;         // Control Flags
   ushort   serv_task;             // Task of the user function to be called after receive or transmit ok
   ushort   serv_func;             // Task of the user function to be called after receive or transmit ok
} M049MsgType;
#ifdef _WIN32
   #pragma pack(pop, SORCPACK)
#else
   #pragma -a
#endif

/****************************************************************************/
// Definitions that should be used by the user program:

#define M049TASK_PROG_NO     0x410     // program-number
#define M049TASK_FLAGS       0x0F8A    // program-install-flags
#define M049_SRQ_SERVICE_FUNC    14    // Number of the srq-service-function in m049task

// Defines for Control Flags of a messageobject
#define _M049_EXCL_BUFFER    0x01
#define _M049_SEND           0x02
#define _M049_ACTIVE         0x08
#define _M049_PASSIVE        0x10
#define _M049_DISABLE_INT    0x20      // new in version 2.A

/****************************************************************************/
// Error message codes:
// If a function of M049TASK detects an error it will return the value
// 0x42E0 to the calling library function. The same value will be returned
// to the user program that called the M049_LIB function. There are some
// library functions that check for errors before calling the M049TASK
// function. These ones can return another error code than 0x42E0. The meaning
// of the return value can be found in the MODULAR-4/486 manual. The
// value can be different in RT or PC programs.

#define M049_TASK_ERROR      0x42E0 // general error code for all library functions
#define M049_INSTALL_ERROR   0x18E0 // the program m049task.exe has not been installed correctly
// The next error codes are different for pc- and rt-programs:
#ifdef ML8RT
   #define M049_INVALID_SLOT 0x1BE0 // the parameter micro_slot specifies an empty slot
   #define M049_WRONG_PAR    0x16E0 // one parameter has an invalid value
   #define M049_FUNC_NOT_IMPLEMENTED 0x08E0 // call of a driver procedure which is
                                    // not implemented in the installed driver version
#else
   #define M049_INVALID_SLOT 0x0226 // the parameter micro_slot specifies an empty slot
   #define M049_WRONG_PAR    0x0228 // one parameter has an invalid value
   #define M049_FUNC_NOT_IMPLEMENTED 0x0223 // call of a driver procedure which is
                                    // not implemented in the installed driver version
#endif

/****************************************************************************/
// prototypes for the m049_lib library

#if defined(__cplusplus)
extern "C" {
#endif

ML10API ushort MLXEXPORT  m049_bib_startup (void);
ML10API ushort MLXEXPORT  m049_config_module (byte micro_slot, M049ConfigType *config);
ML10API ushort MLXEXPORT  m049_config_msg_object (byte micro_slot, M049Handle *msg_handle, M049MsgType *config);
ML10API ushort MLXEXPORT  m049_start_com (byte micro_slot);
ML10API ushort MLXEXPORT  m049_stop_com (byte micro_slot);
ML10API ushort MLXEXPORT  m049_set_data (byte micro_slot, M049Handle msg_handle, byte *data, byte size);
ML10API ushort MLXEXPORT  m049_get_data (byte micro_slot, M049Handle msg_handle, byte *data_var, byte *size_var);
ML10API ushort MLXEXPORT  m049_send_data (byte micro_slot, M049Handle msg_handle);
ML10API ushort MLXEXPORT  m049_send_data_request (byte micro_slot, M049Handle msg_handle);
ML10API ushort MLXEXPORT  m049_clear_msg (byte micro_slot, M049Handle msg_handle);
ML10API ushort MLXEXPORT  m049_set_acceptance_mask (byte micro_slot, ulong *mask, byte usage,
                                                 ushort serv_task, ushort serv_func);
ML10API ushort MLXEXPORT  m049_get_id (byte micro_slot, M049Handle msg_handle, ulong *id);
ML10API void   MLXEXPORT  m049_set_dout (byte micro_slot, byte nr, byte onoff);
ML10API byte   MLXEXPORT  m049_get_dout (byte micro_slot, byte nr);
ML10API ushort MLXEXPORT  m049_get_diagnosis (byte micro_slot);
ML10API ushort MLXEXPORT  m049_evaluate_srq (byte *micro_slot, M049Handle *handle);
ML10API ushort MLXEXPORT  m049_clear_configuration (byte micro_slot);
ML10API void   MLXEXPORT  m049_end_of_srq (byte micro_slot);
ML10API void   MLXEXPORT  m049_enable_int_mask (byte micro_slot);
ML10API ushort MLXEXPORT  m049_get_msg_count (byte micro_slot, M049Handle handle, ulong *count);
ML10API void   MLXEXPORT  m049_init_internal(void);
ML10API ushort MLXEXPORT  m049_get_task_number(byte micro_slot);
ML10API ushort MLXEXPORT  m049_create_msg_object(byte micro_slot, M049Handle *handle,
												ulong id, byte control_flags,
												ushort serv_task, ushort serv_func);
#if defined(__cplusplus)
}                                  // End of block extern "C"
#endif

#endif                             // End of ifndef _m049_lib_
