/***************************************************************************
* Program M049PC01.C to demonstrate the use of the M049LIB PC-library
* for the M-CAN-1 module. M049TASK.EXE must be Version 2.A or higher.
*
* Version: 1.B.        Created: 22.02.1999
*                      by: JD
*
* This sample-program shows how to use the M-CAN-1 module in a PC-program.
* It demonstrates the configuration of the module. It shows how to create
* message-objects of the different types and how to use them.
*
* This program was tested with the Borland C++ 3.1 compiler.
***************************************************************************/

#include "ml8bib.h"
#include "m049_lib.h"
#include "dos.h"
#include "conio.h"
#include "bios.h"

////////////////////////// Function Prototypes \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void EXPORT msg_service (void);
void EXPORT errorhandler(void);
byte BinToBcd (byte bin);
ushort display_error (ushort error, char *text);
void display_help(void);
ushort ProgStart (void);
ushort MsgObjConfiguration (void);

//////////////////////////// Global Variables \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

struct ml8_error_info_type e;
byte         micro_slot = 3;
ushort		 tasknum = 0x23;      // task number of M049TASK.EXE
ushort		 ServTaskNum = 0x25;  // task number of M8P0412.EXE
ushort		 error_status = 0;
ushort       error_in_get_id = 0;
ushort       error_in_get_data = 0;
ushort		 srq_status = 0;
ushort       srq_err = 0;
ulong 		 srq_id = 0;
byte			 srq_data[8];
byte         srq_data_size = 0;
M049Handle   srq_h = 0;
M049Handle   ActSndObjTime, PasSndObjCount, ActSndObjNoIrq;
M049Handle   ActRcvObj1, PasRcvObj1, PasRcvObj2, ActRcvObjGetID;
M049Handle   ActRcvObjExcl, PasRcvObjExcl, PasSndObjExcl, ActSndObjExcl;

//////////////////////////// Error Handling \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void EXPORT errorhandler()
{
	ml8_get_error_info(&e);
	error_status = 1;
	if (e.errorclass == 5)
		srq_err = ml8_get_message();
	ml8_clear_error();
}

//////////////////////////// SRQ Handling \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void EXPORT msg_service ()
{
	byte slot;

	srq_err = m049_evaluate_srq(&slot, &srq_h);
	if (slot == micro_slot) {
		srq_status++;
		if (srq_h) {
			srq_data_size = 8;
			error_in_get_data = m049_get_data(micro_slot, srq_h, srq_data, &srq_data_size);
			error_in_get_id = m049_get_id (micro_slot, srq_h, &srq_id);
		}
		m049_end_of_srq(micro_slot);
	}
}

// help function
byte BinToBcd (byte bin)
{
	return ((bin%10) | ((bin/10)<<4));
}

//////////////////////////// Help Window \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

void display_help()
{
	window(1, 1, 80, 25);
	clrscr();
	cprintf("\n\r Pressing the following keys will cause activities of the module: \n\r");
	cprintf("\n\r - t: Transmit actual PC time with ID 26h.");
	cprintf("\n\r - r: Request data from a message-object with ID 24h.");
	cprintf("\n\r - i: Card transmits remote-frame with ID 25h. The answer data is used as value");
	cprintf("\n\r      for the ID of a new message-object that will be transmitted by M8P0412.");
	cprintf("\n\r - n: Configure a new receive-object with ID 37h.");
	cprintf("\n\r - a: Enable the reception of message-objects with IDs 34h, 35h, 36h, 37h");
	cprintf("\n\r - c: Clear configuration and configure new message-objects");
	cprintf("\n\r      with IDs 127h and 137h. Transmit with 't' and 'r' then.");
	cprintf("\n\r -0,1 Switch DOUT 0 and transmit the state with ID 96h");
	cprintf("\n\r -2,3 Switch DOUT 1 and write the state in MsgObj with ID 97h");
	cprintf("\n\r - f  Transmit Remote frame with ID 95h.");
	cprintf("\n\r - h: Show this help window");
	cprintf("\n\r - q: Quit program");
}

////////////////////// Error Diagnostic and Display \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

ushort display_error (ushort error, char *text)
{
	if (error) {
		cprintf("\n\r Error %X in %s", error, text);
		if (error == 0x42) {
			error = m049_get_diagnosis(micro_slot);
			cprintf(" (Diagnosis %X)", error);
		}
		ml8_exit();
		return(1);
	}
	error = m049_get_diagnosis(micro_slot);
	if (error)
		cprintf("\n\r Warning %X in %s", error, text);
	return(0);
}

//////////////////////////// Initialization \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

ushort ProgStart ()
{
	M049ConfigType ModulCfg;
	ushort 		   io_adr = 0x380;
	ushort 		   pc_irq = 7;
	ushort 			error = 0;
	char				osx[20];
	struct {
		ushort	prog_no;
		char	   version[2];
	} prog_info;

	clrscr();
	cprintf("\n\r Sample to demonstrate the use of the M-CAN-1 module. (C) SORCUS 1997");
	cprintf("\n\r ********************************************************************");
	cprintf("\n\r (card address %X, PC-interrupt %d, M-CAN-1 in slot %d)",io_adr,pc_irq,micro_slot);
	cprintf("\n\r\n\r For full functionality there must be another CAN-bus-station with");
	cprintf("\n\r the following message-objects:");
	cprintf("\n\r Identifier | Usage");
	cprintf("\n\r -----------|----------------------------------------------------------");
	cprintf("\n\r    02h     | Receive data");
	cprintf("\n\r    24h     | Send data on request");
	cprintf("\n\r    25h     | Send 2 data byte [02],[00] on request");
	cprintf("\n\r    26h     | Receive data");
	cprintf("\n\r    27h     | Request data");
	cprintf("\n\r 34h to 37h | Send data");
	cprintf("\n\r    94h     | Send data");
	cprintf("\n\r    95h     | Send data on request");
	cprintf("\n\r    96h     | Receive data");
	cprintf("\n\r    97h     | Request data");
	cprintf("\n\r    127h    | Receive data");
	cprintf("\n\r    137h    | Request data");
	cprintf("\n\r\n\r   Wait...");
	window(40, 19, 80, 25);

	// Initialize ML8 library
	ml8_bib_startup(LANGUAGE_C);
	// reset ml8 and initialize
	error = ml8_reset(0,io_adr,1,7,0,errorhandler,"ROM",20);
//	error = ml8_start(0,io_adr,1,7,0,errorhandler,20);
//	error = ml8_reset(0,io_adr,1,7,0,errorhandler,"c:\\sorcus\\ml8\\osx\\ml8-*.*",20);
	cprintf("\n\r -------------------------------------");
	ml8_get_os_version(osx);
	cprintf("\n\r | OsX = %s",osx);
	if (error) {
		cprintf("\n\r Error %X reseting ML8",error);
		ml8_exit();
		return(1);
	}
	ml8_get_bib_version(osx);
	cprintf("\n\r | ML8BIB-Version = %s",osx);

	// set SRQ-Service-function
	ml8_set_service(msg_service);
	// Install Driver-Program M049TASK under Irq F
	error = ml8_transfer_and_install("..\\..\\..\\..\\m049task\\m049task.exe",
												 0, tasknum,
												 M049TASK_PROG_NO, 0x79,
												 M049TASK_FLAGS, 0);
	if (error) {
		cprintf("\n\r Error %X installing M049TASK", error);
		ml8_exit();
		return(1);
	}

	ml8_get_task_info(tasknum,4,(ulong*)&prog_info);
	cprintf("\n\r | M049TASK Version %c.%c installed",prog_info.version[0],prog_info.version[1]);
	cprintf("\n\r -------------------------------------");
	if (prog_info.version[0] < '2') {
		cprintf("\n\r M049TASK must be Version 2.A or higher, because this program uses new features");
		ml8_exit();
		return(1);
	}
	cprintf("\n\r   Press key to continue...");
	// install program m8p0412 as a message service task
	error = ml8_transfer_and_install("..\\..\\..\\rt\\c\\beisp\\m8p0412.exe",
												0, ServTaskNum, 0x412, 0, 0x983, 0);
	if (error) {
		cprintf("\n\r Error %X installing M8P0412", error);
		ml8_exit();
		return(1);
	}
	// Initialize parameter of m8p0412.exe
	ml8_write_par_byte(ServTaskNum,0,micro_slot);

	// Initialize module library
	error = m049_bib_startup();
	if (display_error(error, "m049_bib_startup"))
		return(1);

	// config mudule
	ModulCfg.messages = 9;       // no significance if M049TASK >= V.2 Rev A
	ModulCfg.serv_task = tasknum;
	ModulCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_module (micro_slot, &ModulCfg);
	if (display_error(error, "m049_config_module"))
		return(1);
	return(0);
}

//////////////////////// Message Object Configuration \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

ushort MsgObjConfiguration ()
{
	M049MsgType    MsgCfg;
	ushort 			error = 0;

	// ** ACTIVE TRANSMIT OBJECT **
	// This is a transmit-object that transmits its data only by a call of
	// m049_send_data.
	MsgCfg.id = 0x26;
	MsgCfg.control_flags = _M049_ACTIVE + _M049_SEND;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &ActSndObjTime, &MsgCfg);
	if (display_error(error, "m049_config_msg_object ActSndObjTime"))
		return(1);

	// ** PASSIVE TRANSMIT OBJECT **
	// This is a transmit-object that transmits its data only by request.
	// The actual data will be transmitted when it receives a remote request.
	// Another CAN-bus station has to send a remote frame to get the data.
	// It is adviceable to use _M049_EXCL_BUFFER for such objects.
	MsgCfg.id = 0x27;
	MsgCfg.control_flags = _M049_PASSIVE + _M049_SEND + _M049_EXCL_BUFFER;
	MsgCfg.serv_task = ServTaskNum;
	MsgCfg.serv_func = 4;
	error = m049_config_msg_object (micro_slot, &PasSndObjCount, &MsgCfg);
	if (display_error(error, "m049_config_msg_object PasSndObjCount"))
		return(1);

	// ** ACTIVE RECEIVE OBJECT **
	// This is a receive-object that requests data by sending a remote frame
	// only by a call of m049_send_data_request.
	MsgCfg.id = 0x24;
	MsgCfg.control_flags = _M049_ACTIVE;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &ActRcvObj1, &MsgCfg);
	if (display_error(error, "m049_config_msg_object ActRcvObj1"))
		return(1);

	// ** PASSIVE RECEIVE OBJECT **
	// This is a receive-object that receives data frames whenever they are on
	// the bus. Another CAN-bus station has to send the data.
	MsgCfg.id = 0x36;
	MsgCfg.control_flags = _M049_PASSIVE;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &PasRcvObj1, &MsgCfg);
	if (display_error(error, "m049_config_msg_object PasRcvObj1"))
		return(1);

	// Message Object with a message service function in RT-program m8p0412.
	// config active receive object with id 25h
	MsgCfg.id = 0x25;
	MsgCfg.control_flags = _M049_ACTIVE;
	MsgCfg.serv_task = ServTaskNum;
	MsgCfg.serv_func = 2;
	error = m049_config_msg_object(micro_slot,&ActRcvObjGetID,&MsgCfg);
	if (display_error(error, "m049_config_msg_object ActRcvObjGetID"))
		return(1);

	// ** ACTIVE TRANSMIT OBJECT without interrupt reaction ** //
	// This is always ACTIVE and PASSIVE.
	// It is adviceable to use _M049_EXCL_BUFFER for such objects.
	MsgCfg.id = 0x96;
	MsgCfg.control_flags = _M049_ACTIVE + + _M049_PASSIVE +_M049_SEND
								  + _M049_EXCL_BUFFER + _M049_DISABLE_INT;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &ActSndObjExcl, &MsgCfg);
	if (display_error(error, "m049_config_msg_object ActSndObjExcl"))
		return(1);

	// ** PASSIVE TRANSMIT OBJECT with exclusive buffer **
	MsgCfg.id = 0x97;
	MsgCfg.control_flags = _M049_PASSIVE + _M049_SEND + _M049_EXCL_BUFFER;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &PasSndObjExcl, &MsgCfg);
	if (display_error(error, "m049_config_msg_object PasSndObjExcl"))
		return(1);

	// ** ACTIVE RECEIVE OBJECT with exclusive buffer **
	MsgCfg.id = 0x95;
	MsgCfg.control_flags = _M049_ACTIVE + _M049_EXCL_BUFFER;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &ActRcvObjExcl, &MsgCfg);
	if (display_error(error, "m049_config_msg_object ActRcvObjExcl"))
		return(1);

	// ** PASSIVE RECEIVE OBJECT with exclusive buffer **
	MsgCfg.id = 0x94;
	MsgCfg.control_flags = _M049_PASSIVE + _M049_EXCL_BUFFER;
	MsgCfg.serv_task = tasknum;
	MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
	error = m049_config_msg_object (micro_slot, &PasRcvObjExcl, &MsgCfg);
	if (display_error(error, "m049_config_msg_object PasRcvObjExcl"))
		return(1);
	return(0);
}

//////////////////////////// Main Program \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

int main()
{
	M049MsgType    MsgCfg;
	struct time    Data;
	byte           i, l;
	ulong          acc_mask;
	ushort 			error = 0;
	short int		key = 0;
	byte				regs[256];
	ushort			outptr, insize;

	if (ProgStart())
		return;
	if (MsgObjConfiguration())
		return;
	getch();
	display_help();

	// enable CAN-communication
	error = m049_start_com (micro_slot);
	if (display_error(error, "m049_start_com"))
		return;
	// enable the feature to suspend the interrupt from the M-CAN-1
	// from the moment of sending a SRQ until m049_end_of_srq is called
	m049_enable_int_mask(micro_slot);
	// set actual data (always after m049_start_com)
	acc_mask = 0;
	error = m049_set_data(micro_slot,PasSndObjCount,(byte*)&acc_mask,sizeof(acc_mask));
	if (display_error(error, "m049_set_data PasSndObjCount"))
		return;

while (1) {
	if (bioskey(1)) {
		key = bioskey(0);
		switch(key & 0xFF) {

			case 't':   // transmit actual time
							// format: hour, minutes, seconds (each one byte bcd)
				cprintf("\n\r Transmit PC-time with ID 26h...");
				gettime(&Data);
				i = Data.ti_min;
				Data.ti_min = BinToBcd(Data.ti_hour);
				Data.ti_hour = BinToBcd(i);
				Data.ti_hund = BinToBcd(Data.ti_sec);
				Data.ti_sec = 0;
				error = m049_set_data(micro_slot,ActSndObjTime,(byte*)&Data,3);
				if (display_error(error, "m049_set_data ActSndObjTime"))
					return;
				error = m049_send_data(micro_slot, ActSndObjTime);
				if (display_error(error, "m049_send_data ActSndObjTime"))
					return;
				break;

			case 'r':   // transmit remote frame to request data
				cprintf("\n\r Transmit Remote frame with ID 24h...");
				error = m049_send_data_request(micro_slot, ActRcvObj1);
				if (display_error(error, "m049_send_data_request ActRcvObj1"))
					return;
				break;

			case 'n':   // configure a new message object to show the use of
				// dynamical message object configuration.
				cprintf("\n\r Configure passive receive message object with ID 37h...");
				MsgCfg.id = 0x37;
				MsgCfg.control_flags = _M049_PASSIVE;  // passive receive object
				MsgCfg.serv_task = tasknum;
				MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
				error = m049_config_msg_object (micro_slot, &PasRcvObj2, &MsgCfg);
				if (display_error(error, "m049_config_msg_object"))
					return;
				break;

			case 'i':   // transmit remote-request: The service-procedure is in
				// m8p0412. There it will cause the transmission of data frames.				cprintf("\n\r Transmit remote frame with ID 25. Answer is handled by m8p0412");
				error = m049_send_data_request(micro_slot, ActRcvObjGetID);
				if (display_error(error, "m049_send _data_request ActRcvObjGetID"))
					return;
				break;

			case 'a':   // set acceptance mask to receive data frames with
							// ids 34h,35h,36h,37h
				acc_mask = 0xFFC;
				cprintf("\n\r Enable reception of messages with IDs 34h, 35h, 36h, 37h");
				error = m049_set_acceptance_mask(micro_slot, &acc_mask, 0,
															tasknum,M049_SRQ_SERVICE_FUNC);
				cprintf("\n\r Mask value = %X", acc_mask);
				if (display_error(error, "m049_set_acceptance_mask"))
					return;
				break;

			case 'c':   // delete configuration
				cprintf("\n\r Clear configuration and config new message-objects with IDs 127h, 137h.");
				error = m049_clear_configuration(micro_slot);
				if (display_error(error, "m049_clear_configuration"))
					return;
				// Create a new message object (active send) that does not
				// generate an interrupt if it is transmitted.
				MsgCfg.id = 0x127;
				MsgCfg.control_flags = _M049_ACTIVE + _M049_PASSIVE + _M049_SEND
											  + _M049_EXCL_BUFFER + _M049_DISABLE_INT;
				MsgCfg.serv_task = tasknum;
				MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
				error = m049_config_msg_object (micro_slot, &ActSndObjTime, &MsgCfg);
				if (display_error(error, "m049_config_msg_object ActSndObjTime"))
					return;

				// Create a new message object (active receive)
				MsgCfg.id = 0x137;
				MsgCfg.control_flags = _M049_ACTIVE + _M049_EXCL_BUFFER;
				MsgCfg.serv_task = tasknum;
				MsgCfg.serv_func = M049_SRQ_SERVICE_FUNC;
				error = m049_config_msg_object (micro_slot, &ActRcvObj1, &MsgCfg);
				if (display_error(error, "m049_config_msg_object ActRcvObj1"))
					return;

				MsgCfg.id = 0x25;
				MsgCfg.control_flags = _M049_ACTIVE + _M049_EXCL_BUFFER;
				MsgCfg.serv_task = ServTaskNum;
				MsgCfg.serv_func = 2;
				error = m049_config_msg_object(micro_slot,&ActRcvObjGetID,&MsgCfg);
				if (display_error(error, "m049_config_msg_object ActRcvObjGetID"))
					return;

				// enable CAN-communication
				error = m049_start_com (micro_slot);
				if (display_error(error, "m049_start_com"))
					return;
				break;

			case '0':
			case '1':  // set dout 0 and transmit state with id 96h
				cprintf("\n\r Switch DOUT 0 and transmit the state with ID 96h...");
				i = key - 0x30;  // state of dout '0': on, '1': off
				m049_set_dout(micro_slot,0,i);
				l = m049_get_dout(micro_slot,0);
				error = m049_set_data(micro_slot,ActSndObjExcl,&l, 1);
				if (display_error(error, "m049_set_data ActSndObjExcl"))
					return;
				error = m049_send_data(micro_slot, ActSndObjExcl);
				if (display_error(error, "m049_send_data ActSndObjExcl"))
					return;
				break;

			case '2':
			case '3':
				cprintf("\n\r Switch DOUT 1 and write the state in MsgObj with ID 97h...");
				i = key - 0x30 - 2;
				m049_set_dout(micro_slot,1,i);
				l = m049_get_dout(micro_slot,1);
				error = m049_set_data(micro_slot,PasSndObjExcl,&l, 1);
				if (display_error(error, "m049_set_data PasSndObjExcl"))
					return;
				break;

			case 'f':   // transmit remote frame to request data
				cprintf("\n\r Transmit Remote frame with ID 95h...");
				error = m049_send_data_request(micro_slot, ActRcvObjExcl);
				if (display_error(error, "m049_send_data_request ActRcvObjExcl"))
					return;
				break;

			case 'q':   // end program
				cprintf("\n\r Exit program...");
				m049_stop_com(micro_slot);
				ml8_exit();
				return;

			case 'h':   // help info
				display_help();
				break;

			case 'd':   // enable performance measurement through external led
				ml8_external_led_off();
				ml8_write_par_byte(tasknum,12,1);
				break;
			case 'v':
				error = 0;
				insize = 256;
				ml8_call_func(tasknum, 18, 0, & outptr, insize, &insize, regs, (byte*)&error);
				clrscr();
				cprintf("\n\r Register contents of the can controller \n\r");
				for (error = 0; error < 0x100;error+=0x10) {
					for (outptr=error; outptr < (error + 0x10); outptr++)
						cprintf(" %02X", regs[outptr]); // wirte one line
					cprintf("\n\r");
				}
				break;

			case 'o':
				error = 0;
				insize = 64;
				ml8_call_func(tasknum, 19, sizeof(M049Handle), &ActSndObjTime, insize, &insize, regs, (byte*)&error);
				cprintf("\n\r Content of ActSndObjTime \n\r");
				for (i=0; i < 0x10; i++)
					cprintf(" %02X", regs[i]); // write one line
				cprintf("\n\r");
				for (i=0x10; i < 0x20; i++)
					cprintf(" %02X", regs[i]); // write one line
				break;
		}
	}
	// if event has occured, display it:
	if (srq_status) {
		cprintf("\n\r Error=%X  ",srq_err);
		cprintf(" Handle=% 08lX  ",(ulong)srq_h);
		if (srq_h) {
			cprintf(" id=% 08lX  data=",srq_id);
			for (i=0;i<srq_data_size;i++)
				cprintf("% 02X ", srq_data[i]);
			if (display_error(error_in_get_data, "m049_get_data"))
				return;
			if (display_error(error_in_get_id, "m049_get_id"))
				return;
			srq_h = 0;
		}
		srq_status = 0;
	}
	if (error_status) {
		error_status = 0;
		cprintf("\n\r Errorhandler was called: ");
		if (e.errorclass == 5)
			cprintf(" Error Request %X", srq_err);
		else
			cprintf(" Class %d, Code %d", e.errorclass, e.errorcode);
		ml8_exit();
		return;
	}
 }
}