/***************************************************************************/
/**                                                                       **/
/**  This is a TI-Task to show the use of the M-CAN-1 module under        **/
/**  RT-programs. After installation the auto_init is called.             **/
/**  It includes a message service function for the message object        **/
/**  ReqObj2 from the PC-program in the directory ML8\xxx\C\BEISP.        **/
/**  In this function another message object is configured (active        **/
/**  send object) and the task is activated.                              **/
/**  Then the main procedure of the TI-task is called periodically.       **/
/**  Each time a counter value is transmitted through the CAN-bus.        **/
/**  A successfull transmission of the messages is shown through          **/
/*   switching the local led on the MODULAR-4 card alternating on/off.    **/
/**                                                                       **/
/**  Installation Reference:                                              **/
/**                                                                       **/
/**  program-number   : 0x412                                             **/
/**  task-number      : ??                                                **/
/**  interrupt-source : None                                              **/
/**  data-range       : 0                                                 **/
/**  flags            : 0x0983                                            **/
/**                                                                       **/
/**                     for Borland C++ (Version 3.1)                     **/
/**                                                                       **/
/**                             Version 1.B.                              **/
/**                                                                       **/
/**                (c) 1997-99 SORCUS Computer GmbH Heidelberg            **/
/**                                                                       **/
/***************************************************************************/

#pragma option -r-                     /* register variables not used      */
#pragma option -k                      /* use standard stackframe          */

#include "dos.h"
#include "ml8rtbib.h"                  /* header for ML8-realtime-library  */
#include "m049_lib.h"                  /* header for ML8-realtime-library  */

struct parameter_type {
	tdt_type   	tdt;
	byte			micro_slot;
	ushort     	error;
	ushort		counter;
	ulong       ReqCounter;
	M049Handle  SendObj;
	ulong       intervall;
} parameter;

void EXPORT   Msg1Fkt(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);
void EXPORT   Msg2Fkt(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);
void EXPORT   Msg3Fkt(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);

					 /* Program-Descriptor-Table (PDT)   */
struct pdt_type {
	unsigned char type;             /* type of PDT                      */
	unsigned char lengt;            /* length of PDT-header             */
	unsigned int  proc_num;         /* number of procedures             */
	unsigned int  prg_number;       /* program-number                   */
	unsigned char version;          /* program-version                  */
	unsigned char revision;         /* program revision                 */
	unsigned char cpu_type;         /* processor-type                   */
	unsigned char coproz_type;      /* co-prozessor-type                */
	unsigned char language;         /* programming language             */
	unsigned char prg_type;         /* program-type                     */
	unsigned int  flags;            /* installation-flags               */
	unsigned int  interrupt_num;    /* number of the used interrupt     */
	unsigned long data_adr;         /* start-address of data-range      */
	unsigned long data_size;        /* size of data-range               */
	unsigned long data_min;         /* minimum size of data-range       */
	unsigned long data_max;         /* maximum size of data-range       */
	unsigned long parameter_adr;    /* address of parameter-range       */
	unsigned int  parameter_size;   /* number of parameters             */
	unsigned long hyper_adr;        /* start-address of hypertext-range */
	unsigned int  res1;
	unsigned long res2;
	void 			  (*main_proc)(void);   /* adress of the main-task-procedure */
	void          (*auto_init)(void);   /* adress of Auto-Init procedure     */
	void EXPORT   (*msg1_func)(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);
	void EXPORT   (*msg2_func)(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);
	void EXPORT   (*msg3_func)(ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata);
}pdt;

/***************************************************************************/
/*                          AutoInit                                       */
/* called after installation: this procedure configures msg objects and    */
/*	sends a data request for one msg obj                                    */
/***************************************************************************/

void auto_init(void)
{
	ml8rt_entry();                        // save registers and prepare
	parameter.micro_slot = 3;
	parameter.ReqCounter = 0;
	parameter.intervall = 100;
	parameter.error = m049_bib_startup();
	ml8rt_exit();                           // restore registers
}

/***************************************************************************/
/*                          main_task                                      */
/***************************************************************************/

void main_task(void)
{
	ml8rt_entry();                     // save registers and prepare
	parameter.error = m049_set_data(parameter.micro_slot,parameter.SendObj,
							(byte*)&parameter.counter,sizeof(parameter.counter));
	parameter.error = m049_send_data(parameter.micro_slot,parameter.SendObj);
	ml8rt_exit();            			  // restore registers
}

/****************************************************************************/
/* MsgFkt1 - This message service function is called when the data for the  */
/*           Object with ID 25 from the dos-program is received.            */
/* The received data (2 byte) is used as an ID for a new configured message */
/****************************************************************************/

void EXPORT Msg1Fkt (ushort task, ushort insize, M049Handle *msghandle, ushort outsize, byte *outdata)
{
	ushort buf;
	byte   size;
	M049MsgType cfgpar;

	ml8rt_entry_function();                     // save registers and prepare
	size = sizeof(buf);

	m049_get_data(parameter.micro_slot,*msghandle,(byte*)&buf,&size);
	// configure a msg obj with the id-value taken from the first 2 data-bytes
	// from the received data:
	cfgpar.id = buf;
	cfgpar.control_flags = _M049_ACTIVE + _M049_SEND;
	cfgpar.serv_task = task;
	cfgpar.serv_func = 3;
	parameter.error = m049_config_msg_object(parameter.micro_slot,
														 &parameter.SendObj,&cfgpar);
	// activate this task: from now on the main_task procedure will be called
	ml8rt_wakeup_ti_task(task, 0, 0, parameter.intervall, 1000);
	parameter.counter = 0;
	ml8rt_exit_function(0,0,0);                 // save registers and prepare
}

/****************************************************************************/
/* MsgFkt2 as a service procedure for the SendObj: called after every       */
/* successfull transmission of the obj data: switch led                     */
/****************************************************************************/

void EXPORT Msg2Fkt (ushort task, ushort insize, M049Handle *indata, ushort outsize, byte *outdata)
{
	ml8rt_entry_function();                     // save registers and prepare
	if (parameter.counter % 2)
		ml8rt_local_led_on();
	else
		ml8rt_local_led_off();
	parameter.counter++;
	ml8rt_exit_function(0,0,0);                 // save registers and prepare
}

/****************************************************************************/
/* MsgFkt3 as a service procedure for the Object with ID 27h from the DOS   */
/* example program                                                          */
/****************************************************************************/

void EXPORT Msg3Fkt (ushort task, ushort insize, M049Handle *msghandle, ushort outsize, byte *outdata)
{
	ml8rt_entry_function();                     // save registers and prepare
	parameter.ReqCounter++;
	m049_set_data(parameter.micro_slot, *msghandle, (byte*)&parameter.ReqCounter,
					  sizeof(parameter.ReqCounter));
	ml8rt_exit_function(0,0,0);                 // save registers and prepare
}

/******************************************************************************/
/*                            main                                            */
/******************************************************************************/

void main (void)                       /* PREPARE                          */
{
	pdt.type        = 1;                  /* set PDT-type                     */
	pdt.lengt       = _PDT_LENGTH;        /* length of PDT-header             */
	pdt.proc_num    = 5;                  /* number of procedures             */
	pdt.prg_number  = 0x412;              /* program-number                   */
	pdt.version     = '1';                /* program-version                  */
	pdt.revision    = 'B';                /* programm-revision                */
	pdt.cpu_type    = _486DX;             /* CPU-type                         */
	pdt.coproz_type = _487DX;             /* Co-Prozessor-type                */
	pdt.language    = _CPP31;             /* program-language = C++           */
	pdt.prg_type    = _USER_PRG;          /* program-type: user-program       */
	pdt.flags       = 0;                  /* flags                            */
	pdt.flags       = _TI_TASK + _PDT_INFO_ENABLE + _LOCAL_DATA + _LOCAL_PARAMETER;
	pdt.interrupt_num = 0;      /* interrupt-number                 */
	pdt.data_adr    = 0;                  /* address of data-range            */
	pdt.data_size   = 0;                  /* size of data-range               */
	pdt.data_min    = 0;                  /* min. size of data-range          */
	pdt.data_max    = 0;                  /* max. size of data-range          */
	ml8rt_phys_adr(&parameter, &pdt.parameter_adr);
	pdt.parameter_adr  = pdt.parameter_adr + sizeof(tdt_type);
	pdt.parameter_size = sizeof(parameter) - sizeof(tdt_type);
	pdt.hyper_adr  = 0;                  /* address of hypertext-range       */
	pdt.main_proc  = main_task;          /* address of main procedure        */
	pdt.auto_init  = auto_init;          /* address of auto_init procedure   */
	pdt.msg1_func  = Msg1Fkt;            /* address of handler for message 1 */
	pdt.msg2_func  = Msg2Fkt;            /* address of handler for message 2 */
	pdt.msg3_func  = Msg3Fkt;            /* address of handler for message 2 */
	ml8rt_set_pdt_adr(&pdt);              /* return address of PDT to OS      */
}