(***************************************************************************)
(**                                                                       **)
(**  This is a TI-Task to show the use of the M-CAN-1 module under        **)
(**  RT-programs. After installation the auto_init is called.             **)
(**  It includes a message service function for the message object        **)
(**  ReqObj2 from the PC-program in the directory ML8\xxx\C\BEISP.        **)
(**  In this function another message object is configured (active        **)
(**  send object) and the task is activated.                              **)
(**  Then the main procedure of the TI-task is called periodically.       **)
(**  Each time a counter value is transmitted through the CAN-bus.        **)
(**  A successfull transmission of the messages is shown through          **)
(**  switching the local led on the MODULAR-4 card alternating on/off.    **)
(**                                                                       **)
(**  Installation Reference:                                              **)
(**                                                                       **)
(**  program-number   : 0x412                                             **)
(**  task-number      : ??                                                **)
(**  interrupt-source : None                                              **)
(**  data-range       : 0                                                 **)
(**  flags            : 0x0983                                            **)
(**                                                                       **)
(**                     for Borland Pascal (Version 7.0)                  **)
(**                                                                       **)
(**                             Version 1.B.                              **)
(**                                                                       **)
(**                (c) 1999 SORCUS Computer GmbH Heidelberg               **)
(**                                                                       **)
(***************************************************************************)
{$N+}                                     { Coprozessor aus }
{$E+}                                     { keine Emulation }
{$R-}                                     { kein Range-Check }
{$S-}                                     { kein Stack-Checking }
{$I-}                                     { kein IO-Checking }
{$V-}                                     { Keine strikte Stringbergabe }
{$L+}                                     { Local-Symbols fr Turbo-Debugger }
{$F+}                                     { FAR Aufrufe erzwingen! }
{$M 1024, 0, 0}                           { Stack und Heap auf Minimum }

PROGRAM M8P0412;

USES    m049_lib, ml8rtbib;

TYPE
   PDT_Type = Record
	      Typ           : Byte;     { Typ der Kopftabelle }
	      Lengt         : Byte;     { Lnge der Kopftabelle incl. Prozedurangaben }
	      Proc_Num      : Word;     { Anzahl der Prozeduren }
	      Prg_Number    : Word;     { Nummer des Programms }
	      Version       : Char;     { Version des Programms. Z.B.: A,B,C ... }
	      Revision      : Char;     { Revision des Programms. Z.B.: 1,2,3 ... }
              CPU_Type      : Byte;     { Prozessor-Typ }
	      COProz_Type   : Byte;     { Co-Prozessor-Typ }
	      Language      : Byte;     { Programmiersprache }
	      Prg_Type      : Byte;     { Programm-Typ }
              Flags         : Word;     { Installierungsflags }
	      Interrupt_Num : Word;     { Nummer des benutzten Interrupts }
              Data_Adr      : LongInt;  { Anfangsadresse des Datenbereichs (physikalische Adresse) }
	      Data_Num      : LongInt;  { Gre des Datenbereichs }
	      Data_Min      : LongInt;  { Datenbereich den das Programm mindestens bentigt }
	      Data_Max      : LongInt;  { Maximaler Datenbereich den das Programm verarbeiten kann }
              Parameter_Adr : LongInt;  { Adresse des Parameterbereichs (physikalische Adresse) }
	      Parameter_Num : Word;     { Anzahl der Parameter }
              Hyper_Adr     : LongInt;  { Anfangsadresse des Hypertext (physikalische Adresse) }
	      Res1          : Word;
	      Res2          : LongInt;
              Main          : Pointer;  { Adresse der Haupt-Prozedur (Task-Prozedur) }
	      Auto_Init     : Pointer;  { Adresse der Auto-Init-Prozedur }
              msg1_func     : Pointer;
              msg2_func     : Pointer;
              msg3_func     : Pointer;
   end;

Parameter_type = Record
        TDT       : TDT_Type;
	micro_slot: byte;
	error:      word;
	counter:    word;
	ReqCounter: LONGINT;
	SendObj:    M049Handle;
	intervall:   LONGINT;
   end;

VAR
   parameter : parameter_type;
   PDT       : PDT_Type;

(***************************************************************************)
(*                          AutoInit                                       *)
(* called after installation: this procedure configures msg objects and    *)
(*	sends a data request for one msg obj                               *)
(***************************************************************************)

Procedure Auto_Init;
begin
	ml8rt_entry;                        {* save registers and prepare *}
	parameter.micro_slot := 3;
	parameter.ReqCounter := 0;
	parameter.intervall := 100;
	parameter.error := m049_bib_startup;
	ml8rt_exit;                         (* restore registers *)
end;

(***************************************************************************)
(*                          main_task                                      *)
(***************************************************************************)

Procedure Main_task;
begin
        ml8rt_entry;
        parameter.error := m049_set_data(parameter.micro_slot,parameter.SendObj,
			   byte(parameter.counter),sizeof(parameter.counter));
	parameter.error := m049_send_data(parameter.micro_slot,parameter.SendObj);
	ml8rt_exit;
end;

(****************************************************************************)
(* MsgFkt1 - This message service function is called when the data for the  *)
(*           Object with ID 25 from the pc-program is received.             *)
(* The received data (2 byte) is used as an ID for a new configured message *)
(****************************************************************************)

PROCEDURE Msg1Fkt (task, insize: word; VAR msghandle: M049Handle;
                   outsize: word; VAR outdata: byte);
var
        buf  : word;
        size : byte;
        cfgpar: M049MsgType;
begin
	ml8rt_entry_function;          (* save registers and prepare *)
	size := sizeof(buf);
	m049_get_data(parameter.micro_slot, msghandle, byte(buf), size);
	(* configure a msg obj with the id-value taken from the first 2 data-bytes
	   from the received data: *)
	cfgpar.id := buf;
	cfgpar.control_flags := _M049_ACTIVE + _M049_SEND;
	cfgpar.serv_task := task;
	cfgpar.serv_func := 3;
	parameter.error := m049_config_msg_object(parameter.micro_slot,
						  parameter.SendObj, cfgpar);
	(* activate this task: from now on the main_task procedure will be called *)
	ml8rt_wakeup_ti_task(task, 0, 0, parameter.intervall, 1000);
	parameter.counter := 0;
	ml8rt_exit_function(0,0,0);    (* save registers and prepare *)
end;

(****************************************************************************)
(* MsgFkt2 as a service procedure for the SendObj: called after every       *)
(* successfull transmission of the obj data: switch led                     *)
(****************************************************************************)

PROCEDURE Msg2Fkt (task, insize: word; VAR msghandle: M049Handle;
                   outsize: word; VAR outdata: byte);
begin
	ml8rt_entry_function;          (* save registers and prepare *)
	if ((parameter.counter AND 1) = 1) then
		ml8rt_local_led_on
	else
		ml8rt_local_led_off;
	Inc(parameter.counter);
	ml8rt_exit_function(0,0,0);    (* save registers and prepare *)
end;

(*************************************************************************)
(* MsgFkt3 as a service procedure for the Object with ID 27h from the    *)
(* pc sample program                                                     *)
(*************************************************************************)

PROCEDURE Msg3Fkt (task, insize: word; VAR msghandle: M049Handle;
                   outsize: word; VAR outdata: byte);
begin
	ml8rt_entry_function;          (* save registers and prepare *)
	Inc(parameter.ReqCounter);
	m049_set_data(parameter.micro_slot, msghandle,
                      byte(parameter.ReqCounter),
		      sizeof(parameter.ReqCounter));
	ml8rt_exit_function(0,0,0);    (* save registers and prepare *)
end;

(******************************************************************************)
(*                            main                                            *)
(******************************************************************************)

begin
   PDT.Typ := 1;                          { Programm-Desriptor-Tabellen-Typ (im Moment wird nur der Typ 1 untersttzt) }
   PDT.Lengt := _PDT_Length;              { Gre des Vorspanns der Kopftabelle (im Moment immer = _PDT_Length = 48) }
   PDT.Proc_Num := 5;                     { Anzahl der Prozeduren }
   PDT.Prg_Number := $412;                { Programm-Nummer }
   PDT.Version := '1';                    { Programm-Version }
   PDT.Revision := 'B';                   { Programm-Revision }
   PDT.CPU_Type := _486DX;                { CPU-Typ }
   PDT.CoProz_Type := _NOCOPROZ;          { Co-Prozessor-Typ }
   PDT.Language := _TP70;                 { Programmiersprache }
   PDT.Prg_Type := _User_Prg;             { Programm-Typ: Anwenderprogramm }
   PDT.Flags := $0;                       { Flags }
   PDT.Flags := _TI_TASK + _PDT_INFO_ENABLE + _LOCAL_DATA + _LOCAL_PARAMETER;
   PDT.Interrupt_Num := 0;                { Interrupt-Nummer fr die das Programm gedacht ist (ungltig, da NI-Task) }
   PDT.Data_Adr := 0;                     { Adresse des Datenbereichs }
   PDT.Data_Num := 0;                     { Gre des Datenbereichs }
   PDT.Data_Min := 0;                     { Minimaler Datenbereich }
   PDT.Data_Max := 0;                     { Maximaler Datenbereich }
					  { Anfangsadresse des Parameterbereichs }
   ml8rt_phys_adr (@Parameter, PDT.Parameter_Adr);
   PDT.Parameter_Adr := PDT.Parameter_Adr + SizeOf(TDT_Type);
					  { Anzahl der Parameter }
   PDT.Parameter_Num := SizeOf(Parameter) - SizeOf(TDT_Type);
   PDT.Hyper_Adr := 0;                    { Adresse des Hypertextes (wird nicht benutzt) }
   PDT.Res1 := 0;
   PDT.Res2 := 0;
   PDT.Main      := @Main_task;           { Adresse der Hauptprozedur }
   PDT.Auto_Init := @Auto_Init;           { Adresse der Auto-Init-Prozedur }
   PDT.msg1_func := @Msg1Fkt;             (* address of handler for message 1 *)
   PDT.msg2_func := @Msg2Fkt;             (* address of handler for message 2 *)
   PDT.msg3_func := @Msg3Fkt;             (* address of handler for message 3 *)
   ml8rt_Set_PDT_Adr (PDT);               { Adresse der PDT im Rckgabe-Record dem Betriebssystem bergeben }
end.
