(******************************************************************************)
(*                                                                            *)
(*     Sample program to demonstrate the use of the M-CAN-1 CAN bus module    *)
(*     for the MODULAR-4/486. It can transmit and receive some can message    *)
(*     objects. It shows how to use nearly all the library functions.         *)
(*     It uses the real time program M8P0412 which can be found in the        *)
(*     directory ML8\RT\BC\BEISP.                                             *)
(*                                                                            *)
(*     To compile this program the files MLXW32.DCU, MLXTYPE.DCU,             *)
(*     M049_LIB.DCU have to be in the same directory or in the dirctory       *)
(*     specified in project/options/directories/conditionals (see delphi-help *)
(*     for more information). The correcponding DLLs MLXW32, MLXTYPE  and     *)
(*     M049_LIB have to be available. Don't forget to install the MLXDRV      *)
(*     NT-system driver for the MODULAR-4/486.                                *)
(*                                                                            *)
(*     This program was tested with delphi 2.0 and 3.0 under Windows NT.      *)
(*                                                                            *)
(*                         Date    03/03/99                                   *)
(*                         Version 1.B                                        *)
(*                         Author  JD                                         *)
(*                                                                            *)
(******************************************************************************)

unit cansampl;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   MLXW32, MLXTYPE, M049_LIB, StdCtrls, Buttons;

type
   TForm1 = class(TForm)
      ResetButton: TButton;
      Dout0Button: TButton;
      Dout1Button: TButton;
      Help: TBitBtn;
      CanMessageBox: TComboBox;
      Messages: TMemo;
      VersionInfo: TMemo;
      LabelSlotEdit: TLabel;
      SlotComboBox: TComboBox;
      procedure ResetAndConfigClick(Sender: TObject);
      procedure SwitchDout0Click(Sender: TObject);
      procedure SwitchDout1Click(Sender: TObject);
      procedure HelpClick(Sender: TObject);
      procedure CanMessageBoxChange(Sender: TObject);
      procedure SlotComboBoxChange(Sender: TObject);
  end;

const
  default_slot = 1;
  sample_version = '1.B';

var
  Form1: TForm1;
  ActSndObjTime, PasSndObjCount, ActSndObjNoIrq : M049Handle;
  ActRcvObj1, PasRcvObj1, PasRcvObj2, ActRcvObjGetID : M049Handle;
  ActRcvObjExcl, PasRcvObjExcl, ActSndObjExcl, PasSndObjDout1 : M049Handle;
  tasknum, ServTaskNum : WORD;
  micro_slot: BYTE;

implementation

{$R *.DFM}

(**************************************************************************)
(* This function checks if the parameter 'error' indicates an error       *)
(* If it does it displays it including reading of error diagnosis for     *)
(* m049_lib functions                                                     *)
(**************************************************************************)

FUNCTION display_error (error: WORD; text : STR80): WORD;
var
   errormsg: str80;
BEGIN
   if (error <> 0) then begin
      errormsg := ' Error ' + IntToHex(error, 2) + ' in ' + text;
      if (error = $42) then begin      { $42 is the error code for an error
                                         detected by a function of m049task.exe }
         error := m049_get_diagnosis(micro_slot);
         errormsg := errormsg + ' (Diagnosis ' + IntToHex(error, 2) + ') ';
      end;
      ShowMessage(errormsg);
      display_error := 1;
   end
   else begin                          { check for warnings from m049task.exe}
      error := m049_get_diagnosis(micro_slot);
      if (error > 1) then begin
         errormsg := ' Warning ' + IntToHex(error, 2) + ' in ' + text;
         ShowMessage(errormsg);
      end;
      display_error := 0;
   end;
end;

(**************************************************************************)
(* This procedure is called if the MLXW32 library detects an error.       *)
(**************************************************************************)

PROCEDURE errorhandler; FAR;
VAR
   errinfo  : ml8_error_info_type;
   errormsg : STR80;
   error    : word;
BEGIN
   ml8_get_error_info(errinfo);
   errormsg := 'Error Handler, Class ' + IntToHex(errinfo.errorclass, 2);
   if (errinfo.errorclass = 5) then begin
      error := ml8_get_message;
      errormsg := errormsg + ', Code ' + IntToHex(error, 2);
   end
   else
      errormsg := errormsg + ', Code ' + IntToHex(errinfo.errorcode, 2);
   ShowMessage(errormsg);
   ml8_clear();
END;

(**************************************************************************)
(* This is the interrupt service procedure called from m049task           *)
(* after a successfull reception or transmission of a message object      *)
(* or in case of a can bus error.                                         *)
(**************************************************************************)

PROCEDURE msg_service; FAR;
VAR
  slot, i   : BYTE;
  srq_error : WORD;
  srq_id    : LONGINT;
  srq_data  : ARRAY[0..7] OF BYTE;
  srq_data_size : BYTE;
  srq_h     : M049Handle;
  srqstr    : STR80;
BEGIN
  srq_error := m049_evaluate_srq(slot, srq_h);
  if (slot = micro_slot) then begin
     if (srq_h <> 0) then begin
      	srq_error := m049_get_id (micro_slot, srq_h, srq_id);
        display_error(srq_error, 'm049_get_id');
     	srq_data_size := 8;
     	srq_error := m049_get_data(micro_slot, srq_h, srq_data[0], srq_data_size);
        display_error(srq_error, 'm049_get_data');
        srqstr := 'Handle  ' + IntToHex(srq_h, 4) + '  ID  ' + IntToHex(srq_id, 4) + '  Data  ';
        if (srq_data_size > 0) then begin
            for i:=0 to (srq_data_size-1) do
                srqstr := srqstr + IntToHex(srq_data[i], 2) + ' ';
        end
        else
            srqstr := srqstr + 'none';
        Form1.Messages.Lines.Add(srqstr);
     end
     else begin
        srqstr := 'CAN Bus Error: ';
        if (srq_error = 1) then
           srqstr := srqstr + 'Bus heavy'
        else
           srqstr := srqstr + 'Bus off';
        Form1.Messages.Lines.Add(srqstr);
     end;
     m049_end_of_srq(micro_slot);
 end;
 END;

(**************************************************************************)
(* In this functions all message objects are configured                   *)
(* It tries to show all combinations of message control flags             *)
(**************************************************************************)

FUNCTION msg_configuration : WORD;
VAR
   MsgCfg : M049MsgType;
   error  : WORD;
BEGIN
   msg_configuration := 0;

   {* ACTIVE RECEIVE OBJECT
      This is a receive-object that requests data by sending a remote frame
      only by a call of m049_send_data_request. *}
   MsgCfg.id := $24;
   MsgCfg.control_flags := _M049_ACTIVE;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, ActRcvObj1, MsgCfg);
   if (display_error(error, 'm049_config_msg_object ActRcvObj1') <> 0) then
      msg_configuration := 1;

   {* ACTIVE RECEIVE OBJECT
      This is a receive-object that requests data by sending a remote frame
      only by a call of m049_send_data_request. The message service function
      is in the RT-program m8p0412. The answer for the remote frame will be
      used as an id value for a new message object that will be configured in
      the message service function *}
   MsgCfg.id := $25;
   MsgCfg.control_flags := _M049_ACTIVE;
   MsgCfg.serv_task := ServTaskNum;
   MsgCfg.serv_func := 2;
   error := m049_config_msg_object(micro_slot,ActRcvObjGetID,MsgCfg);
   if (display_error(error, 'm049_config_msg_object ActRcvObjGetID') <> 0) then
      msg_configuration := 1;

   {* ACTIVE TRANSMIT OBJECT
      This is a transmit-object that transmits its data only by a call of
      m049_send_data. *}
   MsgCfg.id := $26;
   MsgCfg.control_flags := _M049_ACTIVE + _M049_SEND;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, ActSndObjTime, MsgCfg);
   if (display_error(error, 'm049_config_msg_object ActSndObjTime') <> 0) then
      msg_configuration := 1;

   {* PASSIVE TRANSMIT OBJECT
      This is a transmit-object that transmits its data only by request.
      The actual data will be transmitted when it receives a remote request.
      Another CAN-bus station has to send a remote frame to get the data.
      It is adviceable to use _M049_EXCL_BUFFER for such objects. *}
   MsgCfg.id := $27;
   MsgCfg.control_flags := _M049_PASSIVE + _M049_SEND + _M049_EXCL_BUFFER;
   MsgCfg.serv_task := ServTaskNum;
   MsgCfg.serv_func := 4;
   error := m049_config_msg_object (micro_slot, PasSndObjCount, MsgCfg);
   if (display_error(error, 'm049_config_msg_object PasSndObjCount') <> 0) then
      msg_configuration := 1;

   {* PASSIVE RECEIVE OBJECT
      This is a receive-object that receives data frames whenever they are on
      the bus. Another CAN-bus station has to send the data. *}
   MsgCfg.id := $36;
   MsgCfg.control_flags := _M049_PASSIVE;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, PasRcvObj1, MsgCfg);
   if (display_error(error, 'm049_config_msg_object PasRcvObj1') <> 0) then
      msg_configuration := 1;

   {* PASSIVE RECEIVE OBJECT with exclusive buffer *}
   MsgCfg.id := $94;
   MsgCfg.control_flags := _M049_PASSIVE + _M049_EXCL_BUFFER;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, PasRcvObjExcl, MsgCfg);
   if (display_error(error, 'm049_config_msg_object PasRcvObjExcl') <> 0) then
      msg_configuration := 1;

   {* ACTIVE RECEIVE OBJECT with exclusive buffer *}
   MsgCfg.id := $95;
   MsgCfg.control_flags := _M049_ACTIVE + _M049_EXCL_BUFFER;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, ActRcvObjExcl, MsgCfg);
   if (display_error(error, 'm049_config_msg_object ActRcvObjExcl') <> 0) then
   	msg_configuration := 1;

   {* ACTIVE TRANSMIT OBJECT without interrupt reaction
      This is always ACTIVE and PASSIVE.
      It is adviceable to use _M049_EXCL_BUFFER for such objects. *}
   MsgCfg.id := $96;
   MsgCfg.control_flags := _M049_ACTIVE + + _M049_PASSIVE +_M049_SEND
   			+ _M049_EXCL_BUFFER + _M049_DISABLE_INT;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, ActSndObjExcl, MsgCfg);
   if (display_error(error, 'm049_config_msg_object ActSndObjExcl') <> 0) then
      msg_configuration := 1;

   {* PASSIVE TRANSMIT OBJECT with exclusive buffer *}
   MsgCfg.id := $97;
   MsgCfg.control_flags := _M049_PASSIVE + _M049_SEND + _M049_EXCL_BUFFER;
   MsgCfg.serv_task := tasknum;
   MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
   error := m049_config_msg_object (micro_slot, PasSndObjDout1, MsgCfg);
   if (display_error(error, 'm049_config_msg_object PasSndObjDout1') <> 0) then
      msg_configuration := 1;
END;

(**************************************************************************)
(*   This function for selecting the micro slot of the M-CAN-1            *)
(**************************************************************************)

procedure TForm1.SlotComboBoxChange(Sender: TObject);
begin
   micro_slot := SlotComboBox.ItemIndex + 1;
end;

(**************************************************************************)
(*   This function is called when you click the ResetAndConfig Button     *)
(*   It resets the board 0 and configures the can communication           *)
(**************************************************************************)

procedure TForm1.ResetAndConfigClick(Sender: TObject);
var
   error     : WORD;
   irq       : WORD;
   version, date: LONGINT;
   osxstr, datestr: str80;
   prog_info : array[0..3] of char;
   config    : M049ConfigType;
   errormsg  : STR80;
   dout1     : BYTE;
BEGIN
   osxstr     := 'ROM';
   tasknum    := $23;
   ServTaskNum:= $24;
   irq        := $7D;                 { on board interrupt for the M-CAN-1}

   ml8_bib_startup(LANGUAGE_PASCAL);
   error := ml8_reset(0, $380, 1, 7, 0, @errorhandler, osxstr, 20);
   if error <> 0 then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in reset';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;
   error := ml8_transfer_and_install('..\..\..\..\m049task\m049task.exe',
                                      0, tasknum,
                                      M049TASK_PROG_NO, irq,
                                      M049TASK_FLAGS, 0);
   if error <> 0 then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in transfer of m049task';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;

   { get version info }
   ml8_get_osx_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('OsX ' + osxstr + datestr);
   ml8_get_lib_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('Library ' + osxstr + ' ' + datestr);
   ml8_get_mlxdrv_version(version, date);
   ml8_create_version_string(osxstr, version);
   ml8_create_date_string(datestr, date);
   Form1.VersionInfo.Lines.Add('MLXDRV.SYS ' + osxstr + ' ' + datestr);
   ml8_get_task_info(tasknum, 4, LONGINT(prog_info));
   Form1.VersionInfo.Lines.Add('M049Task ' + prog_info[2] + '.' + prog_info[3] +
                               ' installed as task no ' + IntToStr(tasknum));
   Form1.VersionInfo.Lines.Add('M-CAN-1-Settings: Slot: ' + IntToStr(micro_slot)
                                + ' Interrupt: ' + IntToHex(irq, 2));
   Form1.VersionInfo.Lines.Add('Version of this program ' + sample_version);

   ml8_set_service(@msg_service);
   error := m049_bib_startup;
   if error <> 0 then
   begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in m049_bib_startup';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;

   { install program m8p0412 as a message service task }
   error := ml8_transfer_and_install('..\..\..\rt\pascal\sample\m8p0412.exe',
   				     0, ServTaskNum, $412, 0, $983, 0);
   if (error <> 0) then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in transfer of m8p0412';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;
   ml8_write_par_byte(ServTaskNum, 0, micro_slot); { init m8p0412 micro slot }
   error := ml8_read_par_word(ServTaskNum, 1);     { check m8p0412 error status }
   if (error <> 0) then begin
      errormsg := 'Error ' + IntToHex(error, 4) + ' in auto init of m8p0412';
      MessageDlg(errormsg, mtError, [mbYes], 0);
      ml8_exit;
      exit;
   end;

   config.serv_task := tasknum;
   config.serv_func := M049_SRQ_SERVICE_FUNC;
   config.messages  := 0;              { not used by m049task if version >= 2.A }
   error := m049_config_module(micro_slot, config);
   if (display_error(error, 'm049_config_module') <> 0) then
      exit;

   error := msg_configuration;
   if error <> 0 then
      exit;
   error := m049_start_com(micro_slot);
   if (display_error(error, 'm049_start_com') <> 0) then
      exit;

   Dout0Button.Enabled := TRUE;
   Dout1Button.Enabled := TRUE;
   CanMessageBox.Enabled := TRUE;
   Form1.Messages.Visible := TRUE;
   SlotComboBox.Enabled := FALSE;

   { initialize data of PasSndObjDout1 }
   dout1 := m049_get_dout(micro_slot, 1);
   error := m049_set_data(micro_slot, PasSndObjDout1, dout1, 1);
   if (display_error(error, 'm049_set_data PasSndObjDout1') <> 0) then
      exit;

   { initialize data of PasSndObjCount }
   prog_info[0] := #0;
   prog_info[1] := #0;
   prog_info[2] := #0;
   prog_info[3] := #0;
   error := m049_set_data (micro_slot, PasSndObjCount, byte (prog_info[0]), 4);
   if (display_error(error, 'm049_set_data PasSndObjCount') <> 0) then
      exit;

   { enable interrupt locking }
   m049_enable_int_mask(micro_slot);
end;

(**************************************************************************)
(* Function for Dout0 button, it switches digital output 0 of the M-CAN-1 *)
(**************************************************************************)

procedure TForm1.SwitchDout0Click(Sender: TObject);
var dout0: byte;
begin
   dout0 := m049_get_dout(micro_slot, 0);
   if (dout0 = 1) then
      dout0 := 0
   else
      dout0 := 1;
   m049_set_dout(micro_slot, 0, dout0);
end;

(**************************************************************************)
(* Function for Dout1 button, it switches digital output 1 of the M-CAN-1 *)
(* Additionally the data of Message object PasSndObjDout1 (ID 97h) is set  *)
(* to the state of dout 1. Another can bus node can obtain the state of   *)
(* dout1 by sending a remote request frame with ID 97h.                   *)
(**************************************************************************)

procedure TForm1.SwitchDout1Click(Sender: TObject);
var
   error : word;
   dout1 : byte;
begin
   dout1 := m049_get_dout(micro_slot, 1);
   if (dout1 = 1) then
      dout1 := 0
   else
      dout1 := 1;
   m049_set_dout(micro_slot, 1, dout1);
   error := m049_set_data(micro_slot, PasSndObjDout1, dout1, 1);
   if (display_error(error, 'm049_set_data PasSndObjDout1') <> 0) then
      exit;
end;

(**************************************************************************)
(* Function for help button                                               *)
(**************************************************************************)

procedure TForm1.HelpClick(Sender: TObject);
begin
   ShowMessage('For full functionality there must be another CAN-bus-station with the ' + chr(13) +
               'following message-objects:' + chr(13) + chr(13) +
               'Identifier               Usage' + chr(13) +
               '24h                      Send data on request' + chr(13) +
               '25h                      Send 2 data byte on request(used as ID-value for new MsgObj)' + chr(13) +
               '26h                      Receive data (time)' + chr(13) +
               '27h                      Request data (counter value from M8P0412)' + chr(13) +
               '34h to 37h            Send data' + chr(13) +
               '94h                      Send data' + chr(13) +
               '95h                      Send data on request' + chr(13) +
               '96h                      Receive data (state of dout 0)' + chr(13) +
               '97h                      Request data (state of dout 1)' + chr(13) +
               '127h                     Receive data' + chr(13) +
               '137h                     Request data');
end;

(**************************************************************************)
(* Function for the selection box of the user activities                  *)
(**************************************************************************)

procedure TForm1.CanMessageBoxChange(Sender: TObject);
var
   Data  : array[0..7] of byte;
   error : word;
   MsgCfg: M049MsgType;
   acc_mask, count: LONGINT;
   state : byte;
   timestring: str80;
begin
   case CanMessageBox.ItemIndex of
     0: begin { transmit actual time format: hour, minutes, seconds }
        timestring := TimeToStr(time);
        data[0] := (ord(timestring[1]) - $30) * $10;
        data[0] := data[0] + ord(timestring[2]) - $30; { hour }
        data[1] := (ord(timestring[4]) - $30) * $10;
        data[1] := data[1] + ord(timestring[5]) - $30; { minutes }
        data[2] := (ord(timestring[7]) - $30) * $10;
        data[2] := data[2] + ord(timestring[8]) - $30; { seconds }
        error := m049_set_data(micro_slot, ActSndObjTime, data[0], 3);
        if (display_error(error, 'm049_set_data ActSndObjTime') <> 0) then
     	   exit;
        error := m049_send_data(micro_slot, ActSndObjTime);
        if (display_error(error, 'm049_send_data ActSndObjTime') <> 0) then
     	   exit;
        end;
     1: begin { transmit remote frame to request data }
        error := m049_send_data_request(micro_slot, ActRcvObj1);
        if (display_error(error, 'm049_send_data_request ActRcvObj1') <> 0) then
     	   exit;
        end;
     2: begin { transmit remote-request: The service-procedure is in
	        m8p0412. There it will cause the transmission of data frames. }
        error := m049_send_data_request(micro_slot, ActRcvObjGetID);
	if (display_error(error, 'm049_send _data_request ActRcvObjGetID') <> 0) then
     	   exit;
        end;
     3: begin { configure a new message object to show the use of
		dynamical message object configuration. }
	MsgCfg.id := $37;
	MsgCfg.control_flags := _M049_PASSIVE;  // passive receive object
	MsgCfg.serv_task := tasknum;
	MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
	error := m049_config_msg_object (micro_slot, PasRcvObj2, MsgCfg);
	if (display_error(error, 'm049_config_msg_object') <> 0) then
     	   exit;
        end;
     4: begin { set acceptance mask to receive data frames with
		ids 34h,35h,36h,37h }
        acc_mask := $FFC;
	error := m049_set_acceptance_mask(micro_slot, acc_mask, 0,
                                          tasknum,M049_SRQ_SERVICE_FUNC);
        ShowMessage('Acceptance Mask is ' + IntToHex(acc_mask, 4));
	if (display_error(error, 'm049_set_acceptance_mask') <> 0) then
     	   exit;
        end;
     5: begin { transmit remote frame to request data }
	error := m049_send_data_request(micro_slot, ActRcvObjExcl);
        if (display_error(error, 'm049_send_data_request ActRcvObjExcl') <> 0) then
           exit;
        end;
     6: begin { enable permormance measurement }
        ml8_external_led_off();
        ml8_write_par_byte(tasknum, 12, 1);
        end;
     7: begin { get number of interrupts received by message object with id 27h }
        error := m049_get_msg_count (micro_slot, PasSndObjCount, count);
        if (display_error(error, 'm049_get_msg_count PasSndObjCount') <> 0) then
           exit;
        ShowMessage('Message Object with ID 27h has received ' + IntToHex(count,4) + ' messages');
        end;
     8: begin {	transmit state of dout 0 with id 96h }
	state := m049_get_dout(micro_slot,0);
	error := m049_set_data(micro_slot,ActSndObjExcl, state, 1);
	if (display_error(error, 'm049_set_data ActSndObjExcl') <> 0) then
           exit;
	error := m049_send_data(micro_slot, ActSndObjExcl);
	if (display_error(error, 'm049_send_data ActSndObjExcl') <> 0) then
           exit;
        end;
     9: begin {delete configuration and create new message objects with IDs 127h, 137h.}
	error := m049_clear_configuration(micro_slot);
	if (display_error(error, 'm049_clear_configuration') <> 0) then
           exit;
	{ Create a new message object (active send) that does not
         generate an interrupt if it is transmitted. }
         MsgCfg.id := $127;
	 MsgCfg.control_flags := _M049_ACTIVE + _M049_PASSIVE + _M049_SEND
	 		         + _M049_EXCL_BUFFER + _M049_DISABLE_INT;
         MsgCfg.serv_task := tasknum;
         MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
         error := m049_config_msg_object (micro_slot, ActSndObjTime, MsgCfg);
         if (display_error(error, 'm049_config_msg_object ActSndObjTime') <> 0) then
            exit;
	 { Create a new message object (active receive) }
	 MsgCfg.id := $137;
	 MsgCfg.control_flags := _M049_ACTIVE + _M049_EXCL_BUFFER;
	 MsgCfg.serv_task := tasknum;
	 MsgCfg.serv_func := M049_SRQ_SERVICE_FUNC;
	 error := m049_config_msg_object (micro_slot, ActRcvObj1, MsgCfg);
	 if (display_error(error, 'm049_config_msg_object ActRcvObj1') <> 0) then
            exit;
	 MsgCfg.id := $25;
         MsgCfg.control_flags := _M049_ACTIVE + _M049_EXCL_BUFFER;
         MsgCfg.serv_task := ServTaskNum;
         MsgCfg.serv_func := 2;
         error := m049_config_msg_object(micro_slot,ActRcvObjGetID,MsgCfg);
         if (display_error(error, 'm049_config_msg_object ActRcvObjGetID') <> 0) then
            exit;

	 { enable CAN-communication }
         error := m049_start_com (micro_slot);
	 if (display_error(error, 'm049_start_com') <> 0) then
            exit;
         end;
   end;
end;

end.
