Attribute VB_Name = "Module2"
Public ActRcvObj1 As Long      ' id 24
Public ActRcvObjGetID  As Long ' id 25
Public ActSndObjDout0 As Long  ' id 26
Public PasSndObjCount As Long  ' id 27
Public PasRcvObj1 As Long      ' id 36
Public PasRcvObj2 As Long      ' id 37

Public tasknum As Integer
Public ServTaskNum As Integer
Public micro_slot As Byte
Public StateOfDout0 As Byte


'**************************************************************************)
'* This function checks if the parameter 'error' indicates an error       *)
'* If it does it displays it including reading of error diagnosis for     *)
'* m049_lib functions                                                     *)
'**************************************************************************)

Public Function display_error(ByVal error As Integer, ByVal text As String) As Integer
   Dim errormsg As String
   If (error <> 0) Then
      errormsg = "Error " & format(Hex(error), "00") & " in " & text
      If error = M049_TASK_ERROR Then
                             ' detected by a function of m049task.exe
         error = m049_get_diagnosis(micro_slot)
         errormsg = errormsg & " (Diagnosis " & format(Hex(error), "00") & ") "
      End If
      error = MsgBox(errormsg, vbOKOnly, "Error message")
      display_error = 1
   Else                 ' check for warnings from m049task.exe
      error = m049_get_diagnosis(micro_slot)
      If (error > 1) Then
         errormsg = "Warning " & format(Hex(error), "00") & " in " & text
         error = MsgBox(errormsg, vbOKOnly, "Error message")
      End If
      display_error = 0
   End If
End Function

'**************************************************************************)
'* In this functions all message objects are configured                   *)
'* It tries to show all combinations of message control flags             *)
'**************************************************************************)

Public Function msg_configuration() As Integer
   Dim error As Integer
   msg_configuration = 0

   ' ACTIVE RECEIVE OBJECT
   '   This is a receive-object that requests data by sending a remote frame
   '   only by a call of m049_send_data_request. *}
   id = &H24
   control_flags = M049_ACTIVE
   serv_task = tasknum
   serv_func = M049_SRQ_SERVICE_FUNC
   error = m049_create_msg_object(micro_slot, ActRcvObj1, id, _
           control_flags, serv_task, serv_func)
   If (display_error(error, "m049_config_msg_object ActRcvObj1") <> 0) Then
      msg_configuration = 1
   End If

   ' ACTIVE RECEIVE OBJECT
   '  This is a receive-object that requests data by sending a remote frame
   '  only by a call of m049_send_data_request. The message service function
   '  is in the RT-program m8p0412. The answer for the remote frame will be
   '  used as an id value for a new message object that will be configured in
   '  the message service function *}
   id = &H25
   control_flags = M049_ACTIVE
   serv_task = ServTaskNum
   serv_func = 2
   error = m049_create_msg_object(micro_slot, ActRcvObjGetID, id, _
           control_flags, serv_task, serv_func)
   If (display_error(error, "m049_config_msg_object ActRcvObjGetID") <> 0) Then
      msg_configuration = 1
   End If

   ' ACTIVE TRANSMIT OBJECT
   ' This is a transmit-object that transmits its data only by a call of
   ' m049_send_data. *}
   id = &H26
   control_flags = M049_ACTIVE + M049_SEND
   serv_task = tasknum
   serv_func = M049_SRQ_SERVICE_FUNC
   error = m049_create_msg_object(micro_slot, ActSndObjDout0, id, _
           control_flags, serv_task, serv_func)
   If (display_error(error, "m049_config_msg_object ActSndObjDout0") <> 0) Then
      msg_configuration = 1
   End If

   ' PASSIVE TRANSMIT OBJECT
   '  This is a transmit-object that transmits its data only by request.
   '  The actual data will be transmitted when it receives a remote request.
   '  Another CAN-bus station has to send a remote frame to get the data.
   '  It is adviceable to use _M049_EXCL_BUFFER for such objects. *}
   id = &H27
   control_flags = M049_PASSIVE + M049_SEND + M049_EXCL_BUFFER
   serv_task = ServTaskNum
   serv_func = 4
   error = m049_create_msg_object(micro_slot, PasSndObjCount, id, _
           control_flags, serv_task, serv_func)
   If (display_error(error, "m049_config_msg_object PasSndObjCount") <> 0) Then
      msg_configuration = 1
   End If

   ' PASSIVE RECEIVE OBJECT
   '  This is a receive-object that receives data frames whenever they are on
   '  the bus. Another CAN-bus station has to send the data. *}
   id = &H36
   control_flags = M049_PASSIVE
   serv_task = tasknum
   serv_func = M049_SRQ_SERVICE_FUNC
   error = m049_create_msg_object(micro_slot, PasRcvObj1, id, _
           control_flags, serv_task, serv_func)
   If (display_error(error, "m049_config_msg_object PasRcvObj1") <> 0) Then
      msg_configuration = 1
   End If

End Function


Public Sub msg_service()
  Dim srqstr As String
  Dim i As Byte
  Dim srq_error As Integer
  Dim srq_id As Long
  Dim srq_data(8) As Byte
  Dim srq_data_size As Byte
  Dim srq_h As Long
  Dim srq_slot As Byte
  
  srq_error = m049_evaluate_srq(srq_slot, srq_h)
  If (srq_slot = micro_slot) Then
    If (srq_h <> 0) Then
       srq_error = m049_get_id(micro_slot, srq_h, srq_id)
       Call display_error(srq_error, "m049_get_id")
       srq_data_size = 8
       srq_error = m049_get_data(micro_slot, srq_h, srq_data(0), srq_data_size)
       Call display_error(srq_error, "m049_get_data")
       srqstr = "Handle  " & format(Hex(srq_h), "0000") & _
                "  ID  " & format(Hex(srq_id), "0000") & "  Data  "
       If (srq_data_size > 0) Then
           For i = 0 To (srq_data_size - 1) Step 1
               srqstr = srqstr & format(Hex(srq_data(i)), "00") & " "
               Next i
       Else
           srqstr = srqstr & "none"
       End If
    Else
       srqstr = "CAN Bus Error: "
       If (srq_error = 1) Then
          srqstr = srqstr & "Bus heavy"
       Else
          srqstr = srqstr & "Bus off"
       End If
    End If
  End If
  Form1.Text2.text = Form1.Text2.text & vbCrLf & srqstr
  m049_end_of_srq (micro_slot)
End Sub

