(***************************************************************************)
(**                                                                       **)
(**              MODULAR-4 library for SPB module M-CAN-1                 **)
(**                                                                       **)
(**                             Version 2.B                               **)
(**                                                                       **)
(**             Project start    : 11.02.1997                             **)
(**             Last change      : 18.01.1999                             **)
(**                                                                       **)
(**                         Author : JD                                   **)
(**                         Status : tested                               **)
(**                                                                       **)
(**               (c) SORCUS Computer GmbH 1996-99                        **)
(**                                                                       **)
(***************************************************************************)

unit m049_lib;

interface

uses
{$IFDEF WINDOWS}
   {$IFDEF _WIN32}
      mlxw32;          (*Include library for MODULAR-4/486 under Windows 32 Bit*)
   {$ELSE}
      wml8bib;         (*Include library for MODULAR-4/486 under Windows 16 Bit*)
   {$ENDIF}
{$else}
   {$IFDEF ML8RT}
      ml8rtbib;        (*Include library for MODULAR-4/486 realtime programs*)
   {$ELSE}
      ml8bib;          (*Include library for MODULAR-4/486 under DOS*)
   {$ENDIF}
{$ENDIF}

const
   M049TASK_PROG_NO     = $410;
   M049TASK_FLAGS       = $F8A;
   M049_TASK_ERROR      = $42E0;
   M049_INSTALL_ERROR   = $18E0;
   M049_SRQ_SERVICE_FUNC = 14;

{$IFDEF ML8RT}
   M049_INVALID_SLOT    = $1BE0;
   M049_WRONG_PAR       = $16E0;
   M049_FUNC_NOT_IMPLEMENTED = $08E0;
{$ELSE}
   M049_INVALID_SLOT    = $0226;
   M049_WRONG_PAR       = $0228;
   M049_FUNC_NOT_IMPLEMENTED = $0223;
{$ENDIF}

   _M049_EXCL_BUFFER = 1;
   _M049_SEND           = 2;
   _M049_ACTIVE         = 8;
   _M049_PASSIVE        = $10;
   _M049_DISABLE_INT    = $20;

type

M049Handle = LONGINT;

M049ConfigType = RECORD
   messages    : WORD;
   serv_task   : WORD;
   serv_func   : WORD;
                 END;

M049MsgType    = RECORD
   id            : LONGINT;
   control_flags : BYTE;
   serv_task     : WORD;
   serv_func     : WORD;
                 END;

(* m049_lib_startup ... initializes this modul library. *)
FUNCTION m049_bib_startup : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_config_module ... configures a modul *)
FUNCTION m049_config_module
   (micro_slot: BYTE; VAR config: M049ConfigType) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_config_msg_object ... configures a new Message object *)
FUNCTION m049_config_msg_object (micro_slot: BYTE;
         VAR handle: M049Handle; VAR config: M049MsgType) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_start_com... starts the can communication*)
FUNCTION m049_start_com (micro_slot: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_stop_com... stops the can communication*)
FUNCTION m049_stop_com (micro_slot: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_set_data... set data in message object*)
FUNCTION m049_set_data (micro_slot: BYTE; handle: M049Handle;
         VAR data: BYTE; size: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_data... get data of one message object data frame*)
FUNCTION m049_get_data (micro_slot: BYTE; handle: M049Handle;
         VAR data: BYTE; VAR size: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_send_data... transmits message object*)
FUNCTION m049_send_data (micro_slot: BYTE; handle: M049Handle) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_send_data_request... transmits message object remote frame*)
FUNCTION m049_send_data_request (micro_slot: BYTE; handle: M049Handle) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_clear_msg... deactivate message object*)
FUNCTION m049_clear_msg (micro_slot: BYTE; handle: M049Handle) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_set_acceptance_mask... filter messages in message buffer 15*)
FUNCTION m049_set_acceptance_mask (micro_slot: BYTE; VAR mask: LONGINT;
         usage: BYTE; serv_task: WORD; serv_func: WORD) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_id... get the id of one message object *)
FUNCTION m049_get_id (micro_slot: BYTE; handle: M049Handle;
                      VAR id_var: LONGINT) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_diagnosis... get error diagnosis *)
FUNCTION m049_get_diagnosis(micro_slot: BYTE): WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_evaluate_srq... evaluates service request*)
FUNCTION m049_evaluate_srq(VAR micro_slot: BYTE; VAR handle: M049Handle): WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_dout... read state of one digital output *)
FUNCTION m049_get_dout (micro_slot: BYTE; nr: BYTE): BYTE;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_set_dout... set one digital output *)
PROCEDURE m049_set_dout (micro_slot, dout, data: BYTE);
{$ifdef _WIN32} stdcall; {$endif}

(* m049_clear_configuration... clear message object configuration*)
FUNCTION m049_clear_configuration(micro_slot: BYTE): WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_end_of_srq ... allow interrupt from module *)
PROCEDURE m049_end_of_srq(micro_slot: BYTE);
{$ifdef _WIN32} stdcall; {$endif}

(* m049_enable_int_mask ... allow suspending of interrupt from module *)
PROCEDURE m049_enable_int_mask (micro_slot: BYTE);
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_msg_count.. get the number of interrupts of one message object *)
FUNCTION m049_get_msg_count (micro_slot: BYTE; handle: M049Handle;
                             VAR count: LONGINT) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_get_task_number.. get the number of m049task for this micro_slot *)
FUNCTION m049_get_task_number (micro_slot: BYTE) : WORD;
{$ifdef _WIN32} stdcall; {$endif}

(* m049_init_internal.. instead of m049_bib_startup if micro_slot=task number of M049TASK *)
PROCEDURE m049_init_internal;
{$ifdef _WIN32} stdcall; {$endif}


implementation

{$ifndef WINDOWS}
   {$L m049_lib.obj}
{$endif}

FUNCTION m049_bib_startup : WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_config_module (micro_slot: BYTE; VAR config: M049ConfigType)
         : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_config_msg_object (micro_slot: BYTE; VAR handle: M049Handle;
         VAR config: M049MsgType) : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_start_com (micro_slot: BYTE) : WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_stop_com (micro_slot: BYTE) : WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_set_data (micro_slot: BYTE; handle: M049Handle; VAR data: BYTE;
         size: BYTE) : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_data (micro_slot: BYTE; handle: M049Handle; VAR data: BYTE;
         VAR size: BYTE) : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_send_data (micro_slot: BYTE; handle: M049Handle) : WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_send_data_request (micro_slot: BYTE; handle: M049Handle) : WORD;
         EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_clear_msg (micro_slot: BYTE; handle: M049Handle) : WORD;  EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_set_acceptance_mask (micro_slot: BYTE; VAR mask: LONGINT; usage: BYTE;
         serv_task: WORD; serv_func: WORD) : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_id (micro_slot: BYTE; handle: M049Handle; VAR id_var: LONGINT)
         : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_diagnosis(micro_slot: BYTE): WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_evaluate_srq(VAR micro_slot: BYTE; VAR handle: M049Handle): WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
PROCEDURE m049_set_dout (micro_slot, dout, data: BYTE); EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_dout (micro_slot: BYTE; nr: BYTE): BYTE; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_clear_configuration(micro_slot: BYTE): WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
PROCEDURE m049_end_of_srq(micro_slot: BYTE); EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
PROCEDURE m049_enable_int_mask (micro_slot: BYTE); EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_msg_count (micro_slot: BYTE; handle: M049Handle; VAR count: LONGINT)
         : WORD; EXTERNAL {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
FUNCTION m049_get_task_number (micro_slot: BYTE) : WORD; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};
PROCEDURE m049_init_internal; EXTERNAL
         {$ifdef WINDOWS} 'm049_lib.dll'{$ENDIF};

BEGIN
END.
